<?php
/*
Plugin Name: Gengo
Plugin URI: http://jamietalbot.com/wp-hacks/gengo/
Description: Multi-language blogging for WordPress.<br/>Licensed under the <a href="http://www.opensource.org/licenses/mit-license.php">MIT License</a>, Copyright &copy; 2006 Jamie Talbot.
Version: 0.8
Author: Jamie Talbot
Author URI: http://jamietalbot.com/
*/

/*
Gengo - Multi-language blogging for WordPress.
Copyright (c) 2006 Jamie Talbot

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the
Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to
do so, subject to the following conditions:

The above copyright notice and this permission notice shall
be included in all copies or substantial portions of the
Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

// Load some variables for when running as ajax.
if (!defined('ABSPATH')) require_once('../../../wp-blog-header.php');

define ("GENGO_BASE_DIR", "gengo/");
define ("GENGO_DIR", "wp-content/plugins/" . GENGO_BASE_DIR);
define ("GENGO_LANGUAGES_PAGE", "gengo_languages_page.php");
define ("GENGO_TRANSLATIONS_PAGE", "gengo_translations_page.php");
define ("GENGO_SYNONYMS_PAGE", "gengo_synonyms_page.php");
define ("GENGO_EXCLUSIONS_PAGE", "gengo_exclusions_page.php");
define ("GENGO_LANGUAGE_TABLE", $table_prefix . "languages");
define ("GENGO_POST2LANG_TABLE", $table_prefix . "post2lang");
define ("GENGO_SUMMARY_TABLE", $table_prefix . "summaries");
define ("GENGO_CAT2SYN_TABLE", $table_prefix . "cat2syn");
define ("GENGO_SYNBLOCK_TABLE", $table_prefix . "synblocks");
define ("GENGO_SIDEBAR_TITLE_LENGTH", 27);
define ("GENGO_SIDEBAR_SELECT_LENGTH", 18);
define ("GENGO_POSTS_LIMIT", 10);
define ("GENGO_GROUPS_LIMIT", 3);
define ("GENGO_TRANSLATIONS_PAGED", 5);
define ("GENGO_DOMAIN", "gengo");

// You can alter this if you really want to, but I don't recommend it.  For a start all your
// permalinks will change, and your permalink structure may break entirely.  There has been
// no testing on anything other than '+', which works.  So leave it :D  Your other alternative
// is to turn off code-appending entirely.
define ("GENGO_LANGUAGE_DIVIDER", '+');

class Language {
	var $code;
	var $language;
	var $locale;
	var $rtl;

	function Language($code, $language, $locale, $rtl) {
		$this->code = $code;
		$this->language = $language;
		$this->locale = $locale;
		$this->rtl = $rtl;
	}
}

class Gengo {
	var $version;
	var $language_table;
	var $summary_table;
	var $post2lang_table;
	var $cat2syn_table;
	var $synblock_table;
	var $languages;
	var $language_cookie;
	var $language_preference;
	var $language_preference_id;
	var $blog_default_language_id;
	var $default_url_exclusions;

	function Gengo($ajax = false) {
		static $language_preference;
		static $language_preference_id;
		static $languages;
		static $codes2ids;
		static $blog_default_language_id;
		static $viewable_code_string;

		$this->version = 0.8;
		$this->language_table = GENGO_LANGUAGE_TABLE;
		$this->post2lang_table = GENGO_POST2LANG_TABLE;
		$this->summary_table = GENGO_SUMMARY_TABLE;
		$this->cat2syn_table = GENGO_CAT2SYN_TABLE;
		$this->synblock_table = GENGO_SYNBLOCK_TABLE;
		$this->language_cookie = 'gengo_language' . COOKIEHASH;
		$this->default_url_exclusions = array('/wp-content', 'wp-login.php', 'xmlrpc.php', 'wp-pass.php', 'wp-register.php');
		$this->append_urls = get_option('gengo_append_urls');
		$this->site_home = get_settings('home');

		$this->languages = & $languages;
		$this->codes2ids = & $codes2ids;
		$this->language_preference = & $language_preference;
		$this->language_preference_id = & $language_preference_id;
		$this->blog_default_language_id = & $blog_default_language_id;
		$this->viewable_code_string = & $viewable_code_string;

		// Populate a languages array for future use.
		$this->set_defined_languages();
		$this->set_blog_default_language();
		$this->language_preference = array();
		$this->language_preference_id = array();
		$this->strict_links = true;
		$this->viewable_code_string = '';

		if (!$ajax) {
			$this->determine_language();

			// Admin UI.
			if (is_admin()) {
				// Installer.
				add_action('activate_gengo/gengo.php', array(&$this, 'activated'));

				add_action('admin_head', array(&$this, 'admin_head'));
				add_action('admin_menu', array(&$this, 'add_menus'));
				add_action('dbx_post_sidebar', array(&$this, 'post_sidebar'));
				add_action('dbx_page_sidebar', array(&$this, 'post_sidebar'));
				add_action('edit_form_advanced', array(&$this, 'edit_form_advanced'));
				add_action('edit_page_form', array(&$this, 'edit_form_advanced'));
				add_action('admin_footer', array(&$this, 'admin_footer'));

				// User actions.
				add_action('profile_personal_options', array(&$this,'profile_personal_options'));
				add_action('personal_options_update', array(&$this,'personal_options_update'));
				add_action('delete_user', array(&$this,'delete_user'));

				// Manage tables.
				add_filter('manage_posts_columns', array(&$this,'manage_posts_columns'));
				add_action('manage_posts_custom_column', array(&$this,'manage_posts_custom_column'), 10, 2);
				add_filter('manage_pages_columns', array(&$this,'manage_posts_columns'));
				add_action('manage_pages_custom_column', array(&$this,'manage_posts_custom_column'), 10, 2);

				// Post updating.
				add_action('save_post', array(&$this, 'edit_post'));
				add_action('delete_post', array(&$this, 'delete_post'));

				// Categories.
				add_action('add_category', array(&$this, 'add_category'));
				add_action('delete_category', array(&$this, 'delete_category'));

				// Miscellaneous.
				add_action('activity_box_end', array(&$this,'activity_box_end'));
			} else {
				// Links and permalinks.
				add_filter('category_link', array(&$this,'rewrite_category_links'));
				add_filter('author_link', array(&$this,'append_links'));
				add_filter('year_link', array(&$this,'append_links'));
				add_filter('month_link', array(&$this,'append_links'));
				add_filter('day_link', array(&$this,'append_links'));
				add_filter('feed_link', array(&$this,'append_links'));
				add_filter('post_comments_feed_link', array(&$this,'post_comments_feed_link'));
				add_filter('query_vars', array(&$this,'query_vars'));
				add_filter('posts_join', array(&$this,'posts_join'));
				add_filter('posts_where', array(&$this,'posts_where'));
				add_filter('prev_next_post_join', array(&$this,'posts_join'));
				add_filter('prev_next_post_where', array(&$this,'posts_where'));
				add_action('parse_query', array(&$this,'parse_query'));
				add_filter('query_string', array(&$this,'query_string'));

				// Semantic information.
				add_action('wp_head', array(&$this,'wp_head'));
				add_filter('the_content', array(&$this,'the_content'));
			}

			// Rewrite rules.
			add_filter('rewrite_rules_array', array(&$this,'everybody_wants_prosthetic_rewrite_rules'));

			// Language appending throughout the site.
			add_filter('post_link', array(&$this,'post_link'), 100, 2);
			add_filter('page_link', array(&$this,'page_link'), 100, 2);

			add_filter('list_cats', array(&$this,'list_cats'), 8, 2);
			add_filter('get_category', array(&$this,'get_category'));
			//add_filter('category_description', array(&$this,'category_description'), 10, 2);

			// Miscellaneous
			add_action('init', array(&$this,'init'));
			add_action('plugins_loaded', array(&$this,'plugins_loaded'), 0);
		}
	}

	// Filters and Actions.
	// Send the user straight to the configuration page after they activated the plugin.
	function activated() {
		header('Location: admin.php?page=' . GENGO_BASE_DIR . GENGO_LANGUAGES_PAGE);
		die();
	}

	// Add the Gengo options page to the menu.
	function add_menus() {
		add_menu_page(__('Gengo Options', GENGO_DOMAIN), __('Gengo', GENGO_DOMAIN), 8, GENGO_DIR . GENGO_LANGUAGES_PAGE);
    add_submenu_page(GENGO_DIR . GENGO_LANGUAGES_PAGE, __('Gengo Languages', GENGO_DOMAIN), __('Languages', GENGO_DOMAIN), 8, GENGO_DIR . GENGO_LANGUAGES_PAGE);
    add_submenu_page(GENGO_DIR . GENGO_LANGUAGES_PAGE, __('Gengo Translations', GENGO_DOMAIN), __('Translations', GENGO_DOMAIN), 8, GENGO_DIR . GENGO_TRANSLATIONS_PAGE);
    add_submenu_page(GENGO_DIR . GENGO_LANGUAGES_PAGE, __('Gengo Synonyms', GENGO_DOMAIN), __('Synonyms', GENGO_DOMAIN), 8, GENGO_DIR . GENGO_SYNONYMS_PAGE);
		if (($this->append_urls && !(($_POST['gengo_action'] == "update_appending") && !isset($_POST['gengo_append_urls']))) || (isset($_POST['gengo_append_urls']))) add_submenu_page(GENGO_DIR . GENGO_LANGUAGES_PAGE, __('Gengo Exclusions', GENGO_DOMAIN), __('Exclusions', GENGO_DOMAIN), 8, GENGO_DIR . GENGO_EXCLUSIONS_PAGE);
	}
	
	function admin_head() {
		?>
		<script type="text/javascript">
		var script_uri = "<?php echo $this->site_home . '/' . GENGO_DIR . 'gengo.php'; ?>";
		<?php
		if ($count_languages = count($this->languages)) {
			?>
			var languages = new Array (<?php echo $count_languages ?>);
			var language_names = new Array (<?php echo $count_languages ?>);
			var rtl = new Array (<?php echo $count_languages ?>);
			var clashes = new Array (<?php echo $count_languages ?>);
			<?php
			$i = 0;
			foreach ($this->languages as $language_id => $language_object) {
				?>
				languages[<?php echo $i++ ?>] = <?php echo $language_id ?>;
				language_names[<?php echo $language_id ?>] = '<?php echo $language_object->language ?>';
				rtl[<?php echo $language_id ?>] = '<?php echo ($language_object->rtl) ? "rtl" : "ltr"; ?>';
				clashes[<?php echo $language_id ?>] = 'false';
				<?php
			}
		}
		?>
		var delete_summary_message = '<?php _e("You are about to delete this summary.\\n\'Cancel\' to stop, \'OK\' to delete", GENGO_DOMAIN) ?>';
		var updated_summary_message = '<?php _e("Summary updated.  Your main post was not saved.", GENGO_DOMAIN) ?>';
		var sidebar_title = '<?php _e("Gengo", GENGO_DOMAIN) ?>';
		var sidebar_updating_title = '<?php _e("Gengo (Updating)", GENGO_DOMAIN) ?>';
		var summary_title = '<?php _e('Summary', GENGO_DOMAIN) ?>';
		var readonly_message = '<?php _e('Read Only', GENGO_DOMAIN) ?>';
		var translation_title = '<?php _e('Translation', GENGO_DOMAIN) ?>';
		var translation_updated_message = '<?php _e("Translation updated.  Your main post was not saved.", GENGO_DOMAIN) ?>';
		</script>
		<script type="text/javascript" src="../wp-includes/js/tw-sack.js"></script>
		<script type="text/javascript" src="<?php echo '../' . GENGO_DIR . 'js/gengo_common.js' ?>"></script>
		<script type="text/javascript" src="<?php echo '../' . GENGO_DIR . 'js/gengo_admin.js' ?>"></script>
		<?php
	}

	// Place options on the write page and write post pages.
	function post_sidebar() {
		global $wpdb;

		$post_id = $_GET['post'] ? $_GET['post'] : 0;
		?>
		<fieldset class="dbx-box">
		<h3 class="dbx-handle" id="gengo_sidebar_title"><?php _e('Gengo', GENGO_DOMAIN) ?></h3>
		<div class="dbx-content" id="gengo_sidebar_updating" style="text-align: center; display: none">
		<img src="../wp-content/plugins/gengo/loading.gif" />
		</div>
		<div class="dbx-content" id="gengo_sidebar_group">
		<?php
		if (count($this->languages)) {
			$default_language_id = $this->get_default_language_id();
			if ($post_language_info = $this->get_post_language_data($post_id, 'l.language_id AS post_language_id, p2l.translation_group AS post_translation_group')) {
				extract($post_language_info, EXTR_OVERWRITE);
			} else {
				$post_language_id = 0;
				$post_translation_group = 0;
			}
			?>
			<input type="hidden" id="gengo_language_id" name="gengo_language_id" value="<?php echo $post_language_id ?>" />
			<?php
			if (!$post_language_id) $post_language_id = $default_language_id;
			?>
			<script type="text/javascript">
			var existing_post_id = <?php echo ($_GET['post']) ? $_GET['post'] : 0 ?>;
			var existing_translation_group = <?php echo $post_translation_group ?>;
			</script>
			<label for="gengo_use_language" class="selectit"><?php _e("This article's language:", GENGO_DOMAIN); ?>
			<?php
			// Show a list of defined languages.
			foreach($this->languages as $language_id => $entry) {
				$selected_language = (($post_language_id == $language_id) || (!$post_language_id && ($default_language_id == $language_id))) ? ' selected="selected"' : '';
				$default_list .= ($default_language_id == $language_id) ? "<option value=\"$language_id\"$selected_language>$entry->language (Default)</option>" : "<option value=\"$language_id\"$selected_language>$entry->language</option>";
			}
			?>
			<select name="gengo_use_language" id="gengo_use_language" onchange="gengo_change_language();"><?php echo $default_list; ?></select></label>
			<?php
			$post_summary_group = ($sum = $wpdb->get_var("SELECT summary_group FROM $this->post2lang_table WHERE post_id = $post_id")) ? $sum : 0;
			?>
			<p><label for="gengo_view_translations" class="selectit"><input type="checkbox" name="gengo_view_translations" id="gengo_view_translations" onclick="if(!this.checked){document.getElementById('gengo_translations_container').style.display = 'none';}else{document.getElementById('gengo_translations_container').style.display = 'block';}" /> <?php _e('View/Edit translations', GENGO_DOMAIN) ?></label></p>
			<div id="gengo_translations_container" style="display: none; padding: 0; border-left: 1px solid #ddd; border-top: 1px solid #ddd; border-bottom: 1px solid #ddd;">
				<label for="gengo_translation" class="selectit"><input type="checkbox" name="gengo_translation" id="gengo_translation" onclick="if(!this.checked){document.getElementById('gengo_translation_block').style.display = 'none';document.getElementById('gengo_translation_content_block').style.display = 'none';}else{document.getElementById('gengo_translation_block').style.display = 'block';} gengo_lock_controls(); gengo_refresh_summary_list();"<?php if ($post_translation_group) echo ' checked="checked"'; ?> /> <?php _e('This is a translation', GENGO_DOMAIN) ?></label>
				<input type="hidden" id="gengo_existing_translation_group" name="gengo_existing_translation_group" value="<?php echo $post_translation_group ?>" />
				<div id="gengo_translation_block"<?php if (!$post_translation_group) echo ' style="display: none;"'; ?>>
				<?php $this->generate_translation_lists($post_language_id, $post_translation_group, $post_id) ?>
				</div>
			</div>
			<p style="clear: both;"><label for="gengo_view_summaries" class="selectit"><input type="checkbox" name="gengo_view_summaries" id="gengo_view_summaries" onclick="if(!this.checked){document.getElementById('gengo_summaries_container').style.display = 'none';}else{document.getElementById('gengo_summaries_container').style.display = 'block';}" /> <?php _e('View/Edit summaries', GENGO_DOMAIN) ?></label></p>
			<div id="gengo_summaries_container" style="display: none; padding: 0;">
				<input type="hidden" id="gengo_existing_summary_group" name="gengo_existing_summary_group" value="<?php echo $post_summary_group ?>" />
				<div id="gengo_summary_block" style="padding-top: 0; padding-bottom: 0; border-left: 1px solid #ddd; border-top: 1px solid #ddd; border-bottom: 1px solid #ddd;">
				<?php
				$is_in_group = ($post_translation_group) ? 'true' : 'false';
				$this->generate_summary_lists($post_id, $post_summary_group, $is_in_group, $post_translation_group);
				?>
				</div>
			</div>
			<?php
		} else {
			_e('No languages defined yet.', GENGO_DOMAIN);
		}
		if (current_user_can('configure_gengo')) {
			?><p><a href="admin.php?page=<?php echo GENGO_BASE_DIR . GENGO_LANGUAGES_PAGE ?>"><?php _e('Configure Gengo', GENGO_DOMAIN) ?></a></p><?php
		}
		?>
		</div>
		<?php
	}

	// Place the second translation pane on the posts screen.
	function edit_form_advanced() {
		?>
		<fieldset style="margin: 0; display: none" id="gengo_translation_content_block">
			<div style="float: right" id="gengo_translation_message"></div>
			<div id="gengo_translation_legend"></div>
			<div>
				<textarea onkeydown="document.getElementById('gengo_update_button').style.display = 'block';document.getElementById('gengo_translation_message').innerHTML='<em><?php _e('Alterations have not been saved yet!', GENGO_DOMAIN) ?><em>';" style="width: 100%; margin: 0" rows="10" cols="40" name="gengo_translation_content" id="gengo_translation_content"></textarea>
				<p class="submit">
					<input style="float: right;" id="gengo_cancel_translation_button" name="gengo_cancel_translation_button" type="button" onclick="document.getElementById('gengo_translation_content_block').style.display = 'none';" value="<?php _e('Cancel', GENGO_DOMAIN) ?>" />
					<input style="float: right; font-weight: bold; margin-right: 5px; display: none" id="gengo_update_button" name="gengo_update_button" type="button" onclick="gengo_update_translation_content()" value="<?php _e('Update Translation', GENGO_DOMAIN) ?>" />
				</p>
			</div>
		</fieldset>
		<fieldset style="margin: 0; display: none" id="gengo_summary_content_block">
  		<input type="hidden" name="gengo_summary_id" id="gengo_summary_id" />
  		<input type="hidden" name="gengo_summary_language_id" id="gengo_summary_language_id" />
			<div style="float: right" id="gengo_summary_message"></div>
			<div id="gengo_summary_legend"></div>
			<div>
				<textarea onkeydown="document.getElementById('gengo_summary_update').style.display = 'block';document.getElementById('gengo_summary_message').innerHTML='<em><?php _e('Alterations have not been saved yet!', GENGO_DOMAIN) ?><em>';" style="width: 100%; margin: 0" rows="10" cols="40" name="gengo_summary_content" id="gengo_summary_content"></textarea>
				<p class="submit">
					<input style="float: right;" id="gengo_close_summary_button" name="gengo_close_translation_button" type="button" onclick="document.getElementById('gengo_summary_content_block').style.display = 'none';document.getElementById('gengo_summary_content').value = '';" value="<?php _e('Cancel', GENGO_DOMAIN) ?>" />
					<input name="gengo_summary_update" style="float: right; font-weight: bold; margin-right: 5px; display: none" type="button" onclick="gengo_update_summary(); document.getElementById('gengo_summary_content').value = '';" id="gengo_summary_update" value="<?php _e('Update Summary', GENGO_DOMAIN) ?>" />
				</p>
			</div>
		</fieldset>
		<?php
	}

	// Place delete all translations checkbox on the admin screen.
	function admin_footer() {
		global $post;
		if (preg_match('|post.php|i', $_SERVER['SCRIPT_NAME']) && ("edit" == $_GET['action'])) {
			?>
			<div id="gengo_delete_translations_block"><label for="gengo_delete_all_translations"><input type="checkbox" id="gengo_delete_all_translations" name="gengo_delete_all_translations" /> Delete all translations of this post</label></div>
			<script type="text/javascript">
			var delete_all_message = '<?php printf(__("You are about to delete all translations of the post \'%s\'.\\n\'Cancel\' to stop, \'OK\' to delete", GENGO_DOMAIN), addslashes($post->post_title)) ?>';
			var delete_single_message = '<?php printf(__("You are about to delete the post \'%s\'.\\n\'Cancel\' to stop, \'OK\' to delete", GENGO_DOMAIN), addslashes($post->post_title)) ?>';
			gengo_add_delete_all_checkbox();
			</script>
			<?php
		}
	}

	function remove_orphan_summaries() {
		global $wpdb;

		// Remove any orphaned summaries.
		if ($orphaned_summaries = $wpdb->get_col("SELECT DISTINCT s.summary_group AS summary_group FROM $this->summary_table AS s LEFT JOIN $this->post2lang_table AS p2l ON s.summary_group = p2l.summary_group WHERE p2l.summary_group IS NULL")) {
		  $orphans = implode(',', $orphaned_summaries);
			$wpdb->query("DELETE FROM $this->summary_table WHERE summary_group IN ($orphans)");
		}
	}

	// Handle language information when a post is saved.
	function edit_post($post_id) {
		global $wpdb;
		if(!isset($post_id)) {
			// We are creating a new post.
			$post_id = $_POST['post_ID'];
			$language_id = isset($_POST['gengo_use_language']) ? $_POST['gengo_use_language'] : $this->get_default_language_id();
			$existing_language_id = isset($_POST['gengo_language_id']) ? $_POST['gengo_language_id'] : 0;
		} else {
			// We are editing a post.  Can't support updating language by XMLRPC at the moment.
			$existing_language_id = isset($_POST['gengo_language_id']) ? $_POST['gengo_language_id'] : $wpdb->get_var("SELECT language_id FROM $this->post2lang_table WHERE post_id = $post_id");
			$language_id = isset($_POST['gengo_use_language']) ? $_POST['gengo_use_language'] : $existing_language_id;
		}
		$existing_translation_group = isset($_POST['gengo_existing_translation_group']) ? $_POST['gengo_existing_translation_group'] : 0;
		$translation_id = $_POST['gengo_translation_post'];

		// Get a group number if we are creating a new one.
		if (isset($_POST['gengo_translation'])) {
			if (isset($_POST['gengo_translation_group']) && ($_POST['gengo_translation_group'] == 0) && $translation_id) {
				$new_group = true;
				$translation_group = $wpdb->get_var("SELECT MAX(translation_group) FROM $this->post2lang_table") + 1;
			} else {
				$translation_group = ($_POST['gengo_translation_group']) ? $_POST['gengo_translation_group'] : 0;
			}
		} else {
			$translation_group = 0;
		}

		// If the language and translation hasn't changed, do nothing.
		if ($language_id == $existing_language_id && $translation_group == $existing_translation_group) return $post_id;

		// Insert or update the new or altered data.
		if (!$wpdb->query("UPDATE $this->post2lang_table SET language_id = $language_id, translation_group = $translation_group WHERE post_id = $post_id")) {
			$wpdb->query("INSERT INTO $this->post2lang_table(post_id, language_id, translation_group, summary_group) VALUES ($post_id, $language_id, $translation_group, '')");
		}
		// Check if we now don't need a translation group for the one that was deleted.
		if ($existing_translation_group && ($existing_translation_group != $translation_group)) {
			if ('1' == $wpdb->get_var("SELECT COUNT(*) FROM $this->post2lang_table WHERE translation_group = $existing_translation_group")) {
				$wpdb->query("UPDATE $this->post2lang_table SET translation_group = 0 WHERE translation_group = $existing_translation_group LIMIT 1");
			}
		}

		// Put the other post in the same translation group if we have just created a new one.
		if ($new_group) $wpdb->query("UPDATE $this->post2lang_table SET translation_group = $translation_group WHERE post_id = $translation_id LIMIT 1");

		if (!$summary_group = $wpdb->get_var("SELECT summary_group FROM $this->post2lang_table AS p2l WHERE p2l.translation_group = $translation_group AND p2l.post_id != $post_id LIMIT 1")) {
		  $summary_group = (!$existing_translation_group) ? $wpdb->get_var("SELECT summary_group FROM $this->post2lang_table AS p2l WHERE p2l.post_id = $post_id LIMIT 1") : 0;
		}
		// Make sure the summary group is correct..
		$wpdb->query("UPDATE $this->post2lang_table SET summary_group = $summary_group WHERE post_id = $post_id LIMIT 1");
		$this->remove_orphan_summaries();
		return $post_id;
	}

	// Handle language alterations when a post is deleted.
	function delete_post($post_id) {
		global $wpdb, $current_user;

		if (!$deleted_translation_group = $this->get_post_language_data($post_id, "p2l.translation_group")) {
			$deleted_translation_group = 0;
		}
		if ($_POST['gengo_delete_all_translations']) {
			if ($translations = $this->get_the_translations($post_id)) {
				foreach ($translations as $translation) {
					$translation_id = $translation['post_id'];
					if (!user_can_delete_post($current_user->ID, $translation_id)) continue;

					// Temporarily unregister the action to prevent an infinite loop and delete this translation.
					remove_action('delete_post', array(&$this, 'delete_post'));
					wp_delete_post($translation_id);
					add_action('delete_post', array(&$this, 'delete_post'));
					$post_id .= ", $translation_id";
				}
			}
		}
		$wpdb->query("DELETE FROM $this->post2lang_table WHERE post_id IN ($post_id)");

		if ('1' == $wpdb->get_var("SELECT COUNT(*) FROM $this->post2lang_table WHERE translation_group = $deleted_translation_group")) {
			$wpdb->query("UPDATE $this->post2lang_table SET translation_group = 0 WHERE translation_group = $deleted_translation_group");
		}
		// Remove summaries that have been orphaned.
		$this->remove_orphan_summaries();
	}

	// Add language table columns to the manage posts table.
	function manage_posts_columns(& $post_columns) {
		$post_columns['gengo_language'] = __('Language', GENGO_DOMAIN);
		$post_columns['gengo_translations'] = __('Translations', GENGO_DOMAIN);
		$post_columns['gengo_summaries'] = __('Summaries', GENGO_DOMAIN);
		return $post_columns;
	}

	// Add language information to the manage posts table.
	function manage_posts_custom_column($column_name, $id) {
		if ('gengo_language' == $column_name) {
			echo $this->get_post_language_data($id, 'l.language');
		} elseif ('gengo_translations' == $column_name) {
			if ($translations = $this->get_the_translations($id)) {
				foreach ($translations as $translation) {
					extract($translation, EXTR_OVERWRITE);
					?><a href="<?php echo get_permalink($post_id) ?>" title="<?php printf(__('View the %s translation of this article.', GENGO_DOMAIN), $this->languages[$language_id]->language) ?>"><?php echo "$post_id (" . $this->languages[$language_id]->language . ")" ?><br /></a><?php
				}
			}
		} elseif ('gengo_summaries' == $column_name) {
			$summaries = $this->get_the_summaries($id);
			foreach ($summaries as $summary) echo '<a title="' . $summary['summary'] . '">' . $this->languages[$summary['language_id']]->language . '</a><br />';
		}
	}

  // Modified from original source courtesy of Andy Skelton.  Solved a major headache.
	function everybody_wants_prosthetic_rewrite_rules($rules) {
	  global $wp_rewrite;
		if (!$language_count = count($this->languages)) return $rules;

		foreach ($this->languages as $entry) $codes .= $codes ? '|' . $entry->code : $entry->code;
		for ($i = 0; $i < $language_count; $i++) $code_string .= $i ? "(\+($codes))?" : "($codes)?";

		if ($wp_rewrite->using_index_permalinks()) $index = $wp_rewrite->index . '/';
		$newrules["$index($code_string)/?$"] = 'index.php?language=$matches[1]';
		foreach ($rules as $match => $query) {
			$newmatch = str_replace('/?$', "(?:/$code_string)?/?$", $match);
			preg_match_all('#matches\[(\d+)\]#', $query, $matches);
			$number = count($matches[1]) + 1;
			$newquery = add_query_arg('language', "\$matches[$number]", $query);
			$newrules[$newmatch] = $newquery;
		}
		return $newrules;
	}

	function append_link_language($link, $code, $regular_links = false) {
		global $wp_rewrite;
		$append = $this->append_urls;

		if (!$code || (!$append && !$this->forced_append)) return $link;
		$this->forced_append = false;
		if ((!$query_string = strpos($link, '?')) && get_settings('permalink_structure')) {
			return trailingslashit($link) . "$code/";
		} else {
			if ($query_string) {
				if ($this->strict_links && !$regular_links) {
					return $link . "&amp;language=$code";
				} else {
					return $link . "&language=$code";
				}
			}
			return trailingslashit($link) . "?language=$code";
		}
	}

	// Add language info to page links.
	function page_link($link, $id) {
		global $wpdb;
		if (!$id) return $link;
		return $this->append_link_language($link, $this->languages[$wpdb->get_var("SELECT language_id from $this->post2lang_table WHERE post_id = $id LIMIT 1")]->code);
	}

	// Add language info to the permalink.
	function post_link($link, $post) {
	  return $this->page_link($link, $post->ID);
	}

	// Append language codes and rewrite category links to language synonym.
	function rewrite_category_links($link) {
		global $wp_rewrite, $wpdb, $post;

		if ($wp_rewrite->using_permalinks()) {
			$category_base = $wp_rewrite->category_base ? $wp_rewrite->category_base : 'category';
			$categories = explode('/', substr(rtrim($link, '/'), strpos($link, $category_base) + strlen($category_base) + 1));
			$cat_list = "'" . implode("', '", $categories) . "'";
			$language_id = $this->language_preference_id[0];
			if ($results = $wpdb->get_results("SELECT category_nicename AS original, sanitised FROM $wpdb->categories c INNER JOIN $this->cat2syn_table syn ON c.cat_ID = syn.cat_id WHERE c.category_nicename IN ($cat_list) AND syn.language_id = $language_id", ARRAY_A)) {
				foreach ($results as $result) {
				  if ($result['sanitised']) {
						$originals[] = "/$result[original]/";
						$synonyms[] = "/$result[sanitised]/";
					}
				}
				$link = str_replace($originals, $synonyms, trailingslashit($link));
			}
		}
		return $post->code ? $this->append_link_language($link, $this->viewable_code_string) : $link;
	}

	// Append language codes to other links.
	function append_links($link) {
		return $this->append_link_language($link, $this->viewable_code_string);
	}

	// Append language code to feed links.
	// TODO: This will cause problems in the rare case that a post has the same slug as the language code.
	// TODO: Check post->code;
	function post_comments_feed_link($url) {
	  global $post;
	  if (get_settings('permalink_structure')) $url = str_replace("/{$post->code}/", '/',  $url);
		return $this->append_link_language($url, $post->code);
	}

	// Define the language query variable.
	function query_vars($vars) {
		$vars[] = 'language';
		return $vars;
	}

	// Add language table to the posts query.
	function posts_join($join) {
		global $wpdb;
		if ($this->language_preference) return $join . " INNER JOIN $this->post2lang_table p2l ON $wpdb->posts.ID = p2l.post_id INNER JOIN $this->language_table l ON p2l.language_id = l.language_id";
		return $join;
	}

	// Add language constraint to the posts query.
	function posts_where($original_where) {
		global $wpdb;
		if (($code = $this->language_preference[0]->code) && !is_single() && !is_page()) {
			if (1 == count($this->language_preference)) {
				return $original_where . " AND l.code = '$code'";
			} else {
				foreach ($this->language_preference_id as $key => $language_id) {
					if (0 == $key) $where = "l.language_id = $language_id";
					else $where .= " OR (l.language_id = $language_id AND (translation_group = 0 OR translation_group IN (SELECT translation_group FROM (SELECT translation_group FROM $this->post2lang_table GROUP  BY translation_group HAVING 0 = COUNT(CASE $exclusions END)) AS x$language_id)))";
					$exclusions .= " WHEN language_id = $language_id THEN 1";
				}
				return $original_where . " AND ($where)";
			}
		}
		return $original_where;
	}

	// Translate category synonyms if necessary.
	function parse_query(& $query_object) {
		global $wpdb;
		if (($cat_name = sanitize_title($query_object->query_vars['category_name'])) && ($code = $query_object->query_vars['language'])) {
			foreach ($this->languages as $language_id => $entry) if ($entry->code == $code) break;
			if ($synonym = $wpdb->get_var("SELECT category_nicename FROM $wpdb->categories c INNER JOIN $this->cat2syn_table syn ON c.cat_ID = syn.cat_id WHERE syn.sanitised = '$cat_name' AND syn.language_id = $language_id LIMIT 1")) {
				$query_object->query_vars['category_name'] = $synonym;
			}
		}
	}

	// Checks that the correct language
	function query_string($query_string) {
		global $wpdb;

		parse_str($query_string, $vars);
		if ($vars['p'] || $vars['name'] || $vars['pagename'] || $vars['page_id']) {
			if ($vars['p']) {
				$where = "p2l.post_id = $vars[p]";
			} elseif ($vars['page_id']) {
				$where = "p2l.post_id = $vars[page_id]";
			} elseif ($vars['name']) {
				$where = "p.post_name = '$vars[name]'";
				$join = "INNER JOIN $wpdb->posts AS p ON p.ID = p2l.post_id";
			} else {
				$where = "p.post_name = '$vars[pagename]'";
				$join = "INNER JOIN $wpdb->posts AS p ON p.ID = p2l.post_id";
			}
			if (($language_id = $wpdb->get_var("SELECT language_id FROM $this->post2lang_table AS p2l $join WHERE $where LIMIT 1")) && ($language_id !== $this->language_preference_id[0])) {
			  if ($this->language_supplied) {
			    if ($translation_id = $wpdb->get_var("SELECT p2l2.post_id FROM $this->post2lang_table p2l INNER JOIN $this->post2lang_table p2l2 ON p2l.translation_group = p2l2.translation_group $join WHERE $where AND p2l2.language_id = {$this->language_preference_id[0]}")) {
			      // A translation exists in the language specified, so just go straight there.
						$this->strict_links = false;
						header('Location: ' . get_permalink($translation_id));
						die();
					} else {
				    // There's no translation in the language specified, so just remove it and redirect.
				    if ($_SERVER['QUERY_STRING']) {
							$_SERVER['REQUEST_URI'] = remove_query_arg('language', $_SERVER['REQUEST_URI']);
						} else {
							$request = trailingslashit($_SERVER['REQUEST_URI']);
							$position = strrpos(substr($request, 0, -1), '/');
							$_SERVER['REQUEST_URI'] = substr($request, 0, $position + 1);
						}
					}
				}
				$this->forced_append = true;
				$location = $this->append_link_language($_SERVER['REQUEST_URI'], $this->languages[$language_id]->code, true);
				header("Location: $location");
				die();
			}
		}
		return $query_string;
	}

	// Place default language option on the profile page.
	function profile_personal_options() {
		global $current_user;

		if (count($this->languages)) {
			$default_language_id = $this->get_default_language_id();
			foreach($this->languages as $language_id => $entry) {
				$default_list .= ($default_language_id == $language_id) ? "<option value=\"$language_id\" selected=\"selected\">$entry->language</option>" : "<option value=\"$language_id\">$entry->language</option>";
			}
			?>
			<label for="gengo_default_language"><?php _e('Default language', GENGO_DOMAIN) ?>:</label>
			<select id="language" name="gengo_default_language"><?php echo $default_list; ?></select>
			<?php
		}
	}

	// Handle updated personal default.
	function personal_options_update() {
		global $current_user;
		update_usermeta($current_user->id, 'gengo_default_language', $_POST['gengo_default_language']);
	}

	// Clean up the default language option for a deleted user.
	function delete_user($user_id) {
		global $wpdb;
		$wpdb->query("DELETE FROM $wpdb->usermeta WHERE meta_key = 'gengo_default_language' AND user_id = $user_id");
	}

	function add_language_preference($language_id) {
		$this->language_preference[] = $this->languages[$language_id];
		$this->language_preference_id[] = $language_id;
	}

	function set_language_preference($codes) {
		foreach ($codes as $code) $this->add_language_preference($this->codes2ids[$code]);
	}

	// Save the language preference in a cookie.
	function store_language_preference($original_codes) {
		if ('1' == get_option('gengo_allow_multiread')) {
			if ($existing_cookie = trim($_COOKIE[$this->language_cookie])) {
				$codes = array_unique(array_merge(explode(GENGO_LANGUAGE_DIVIDER, $existing_cookie), $original_codes));
			} else {
				$codes = $original_codes;
			}
		} else {
			$original_codes = $codes = array($original_codes[0]);
		}
		$this->viewable_code_string = (count($codes) > 1) ? implode(GENGO_LANGUAGE_DIVIDER, $codes) : $codes[0];
		setcookie($this->language_cookie, $this->viewable_code_string, time() + 30000000, COOKIEPATH);
		$this->set_language_preference($original_codes);
	}

	// Add semantic links to the head of a page with alternate translations.
	function wp_head() {
		global $wp_query, $wp_rewrite;
		if ($wp_rewrite->using_index_permalinks()) $index = "/$wp_rewrite->index";

		$code = (1 == count($this->language_preference_id)) ? $this->languages[$this->language_preference_id[0]]->code : implode(',', explode(GENGO_LANGUAGE_DIVIDER, $this->viewable_code_string));
		?>
		<meta http-equiv="Content-Language" content="<?php echo $code?>" />
		<script type="text/javascript">
		var add_message = '<?php _e("Add", GENGO_DOMAIN) ?>';
		var up_message = '<?php _e("Up", GENGO_DOMAIN) ?>';
		var down_message = '<?php _e("Down", GENGO_DOMAIN) ?>';
		var x_message = '<?php _e("x", GENGO_DOMAIN) ?>';
		var cookie_name = '<?php echo $this->language_cookie ?>';
		var cookie_path = '<?php echo COOKIEPATH ?>';
		var encoded_divider = '<?php echo urlencode(GENGO_LANGUAGE_DIVIDER) ?>';
		var language_divider = '<?php echo GENGO_LANGUAGE_DIVIDER ?>';
		var save_error_message = '<?php _e("You must choose at least one language to view in!", GENGO_DOMAIN)?>';
		var site_home = '<?php echo $this->site_home . $index ?>';

	  var original_codes = '<?php echo $this->viewable_code_string ?>';
		<?php
		if ($count_languages = count($this->languages)) {
			?>
			var language_codes = new Array (<?php echo $count_languages ?>);
			var language_ids = new Array (<?php echo $count_languages ?>);
			<?php
			foreach ($this->languages as $language_id => $language_object) {
				?>
				language_codes[<?php echo $language_id ?>] = '<?php echo $language_object->code ?>';
				language_ids['<?php echo $language_object->code ?>'] = '<?php echo $language_id ?>';
				<?php
			}
		}
		?>
		</script>
		<script type="text/javascript" src="<?php echo $this->site_home . '/' . GENGO_DIR . 'js/gengo_common.js' ?>"></script>
		<script type="text/javascript" src="<?php echo $this->site_home . '/' . GENGO_DIR . 'js/gengo_site.js' ?>"></script>
		<?php
		if (is_single()) {
			if ($translations = $this->get_the_translations($wp_query->posts[0]->ID)) foreach ($translations as $translation) echo $this->translation_link($translation, 'link');
		} elseif (is_home()) {
			foreach ($this->languages as $language_id => $entry) {
			  if (in_array($language_id, $this->language_preference_id)) continue;
				echo $this->translation_link(array('language_id' => $language_id), 'home_link');
			}
		}
	}

	// Add semantic info to the post content.
	function the_content($content) {
		global $post;
		return "<div xml:lang=\"$post->code\" lang=\"$post->code\">$content</div>";
	}

	// Add language information to a category when a category is added.
	function add_category($id) {
		global $wpdb;
		$language_preference_id = $this->language_preference_id[0];
		$wpdb->query("INSERT INTO $this->cat2syn_table(cat_id, language_id, synonym, sanitised) SELECT $id, $language_preference_id, cat_name, category_nicename FROM $wpdb->categories WHERE cat_ID = $id");
		foreach ($this->languages as $language_id => $entry) if ($language_id != $language_preference_id) $values .= $values ? ", ($id, $language_id, '', '')" : "($id, $language_id, '', '')";
		if ($values) $wpdb->query("INSERT INTO $this->cat2syn_table(cat_id, language_id, synonym, sanitised) VALUES $values");
	}

	// Delete all the synonyms for this category too.
	function delete_category($id) {
		global $wpdb;
		$wpdb->query("DELETE FROM $this->cat2syn_table WHERE cat_id = $id");
	}

	// Translate the categories in lists, if necessary.
	// list_cats is sometimes called with the entire list, which we don't need.
	function list_cats($name, $category = '') {
		global $wpdb;
		if (!$category) return $name;
		$language_id = $this->language_preference_id[0];
		if ($language_id) {
			if ($translated_name = $wpdb->get_var("SELECT synonym FROM $this->cat2syn_table WHERE cat_id = $category->cat_ID AND language_id = $language_id LIMIT 1")) {
				return $translated_name;
			}
		}
		return $name;
	}

	// Function to take advantage of the get_category filter, if it is applied.
	function get_category($category_data) {
	  global $wpdb, $post, $wp_query;
		$language_id = ($wp_query->in_the_loop) ? $post->language_id : $this->language_preference_id[0];
		if ($language_id) {
			if ($translated_name = $wpdb->get_var("SELECT synonym FROM $this->cat2syn_table WHERE cat_id = $category_data->cat_ID AND language_id = $language_id LIMIT 1")) {
				$category_data->cat_name = $translated_name;
			}
		}
		return $category_data;
	}

	// Take advantage of category descriptions.
	function category_description($description, $category) {
		global $wpdb;
		$language_id = $this->language_preference_id[0];
		if ($synonym = $wpdb->get_var("SELECT description FROM $this->cat2syn_table WHERE cat_id = $category->cat_ID AND language_id = $language_id LIMIT 1"))
			return $synonym;
		return $description;
	}

	// Place Gengo information in the activity box on the Dashboard.
	function activity_box_end() {
		global $wpdb;
		if (!$language_count = count($this->languages)) return;
		$blog_default_language = $this->languages[$this->blog_default_language_id]->language;
		$default_language = $this->languages[$this->get_default_language_id()]->language;
		$profile_link = '<a href="' . $this->site_home . '/wp-admin/profile.php#language">' . $default_language . '</a>';
		$language_link = $language_count . ' ';
		if (current_user_can('configure_gengo')) {
			$open = '<a href="admin.php?page=' . GENGO_BASE_DIR . GENGO_LANGUAGES_PAGE . '">';
			$close = '</a>';
		}
		$language_link .= $open . __('language(s)', GENGO_DOMAIN) . $close;
		?>
		<div>
		<h3><?php _e('Gengo'); ?></h3>
		<p><?php printf(__("There are %s defined, of which the blog default is %s and your default is %s.", GENGO_DOMAIN), $language_link, $blog_default_language, $profile_link); ?></p>
		</div>
		<?php
	}

	// TODO: The locale a hack because I can't guarantee that the locale filter will be executed.
	function plugins_loaded() {
		global $locale, $gengo_javascript;
		if (count($this->languages)) {
			// Always present the admin section in the author's default language.
			if (is_admin() || $gengo_javascript) {
				get_currentuserinfo();
				$this->add_language_preference($this->get_default_language_id());
			}
			$locale = $this->language_preference[0]->locale;
		}
		load_plugin_textdomain(GENGO_DOMAIN, GENGO_DIR);
		if (function_exists('register_sidebar_widget') && function_exists('register_widget_control')) {
			register_sidebar_widget('Languages', array(& $this, 'gengo_language_widget'));
			register_widget_control('Languages', array(& $this, 'gengo_language_widget_control'));
		}
	}

	// Define the ability to configure Gengo.
	// TODO: After 2.1, look again at the Role Manager.
	function init() {
		global $wp_roles;
    if (isset($_POST['gengo_uninstall_submit'])) $this->uninstall();
		if (!$wp_roles) return;
		$role = $wp_roles->get_role('administrator');
		$role->add_cap('configure_gengo');
	}

	// Auxilliary Functions.

	function gengo_language_widget($args) {
		extract($args);
		$options = get_option('gengo_widget');
		echo $before_widget;
		echo $before_title . __('Languages', GENGO_DOMAIN) . $after_title;
		?>
		<ul><?php gengo_list_languages($options['show_count']); ?></ul>
		<?php
		echo $after_widget;
	}

	function gengo_language_widget_control() {
		$options = get_option('gengo_widget');
		if ($_POST['gengo_widget_submit']) {
			$updated_options['show_count'] = isset($_POST['gengo_widget_count']) ? 1 : 0;
			if ($options != $updated_options) {
				$options = $updated_options;
				update_option('gengo_widget', $options);
			}
		}
		if ($options['show_count']) $count_checked = 'checked="checked" ';
		?>
		<div>
		<label for="gengo_widget_count" style="line-height:35px; display:block;">Show Post Count: <input type="checkbox" id="gengo_widget_count" name="gengo_widget_count" value="1" <?php echo $count_checked ?>/></label>
		<input type="submit" name="gengo_widget_submit" id="gengo_widget_submit" value="Update Options" />
		</div>
		<?php
	}

	function error_message($message) {
	  ?><div class="error"><p><strong><?php _e('Error: ', GENGO_DOMAIN) ?></strong><?php echo $message ?></p></div><?php
		return false;
	}

	function update_message($message) {
	  ?><div class="updated fade"><p><?php echo $message ?></p></div><?php
		return true;
	}

	function get_synblock($block_name, $language_id = 0) {
		global $wpdb;
		if ($language_id) {
			return $wpdb->get_var("SELECT text FROM $this->synblock_table WHERE block_name = '$block_name' AND language_id = $language_id LIMIT 1");
		} elseif ($synonyms = $wpdb->get_results("SELECT language_id, text FROM $this->synblock_table WHERE block_name = '$block_name'")) {
			foreach ($synonyms as $synonym) {
			  if ($string) $string .= "GENGO_SEPARATOR";
				$string .= $synonym->language_id . "GENGO_SEPARATOR" . $synonym->text;
			}
			echo $string;
		}
	}

	function is_defined_code($code) {
		foreach ($this->languages as $language) if ($language->code == $code) return true;
		return false;
	}

	function is_recognised_code($code) {
		include_once (ABSPATH . GENGO_DIR . 'gengo_languages.php');
		if (isset($gengo_recognised_languages[$code])) return true;
		return false;
	}

	// Returns the order of preference for the given language id, or false if it is not set.
	function is_set_language($language_id) {
		foreach ($this->language_preference_id as $order => $id) if ($id == $language_id) return $order;
		return false;
	}

	// Returns the order of preference for given language, including languages not being displayed.
	function is_viewable_language($language_id) {
		$viewable_codes = explode(GENGO_LANGUAGE_DIVIDER, $this->viewable_code_string);
		foreach ($viewable_codes as $viewable_code) $ids[] = $this->codes2ids[$viewable_code];
		foreach ($ids as $order => $id) if ($id == $language_id) return $order;
		return false;
	}

	function is_excluded_url($url) {
		foreach ($this->default_url_exclusions as $exclusion) if (strstr($url, $exclusion)) return true;
    if (($manual_exclusions = get_option('gengo_url_exclusions')) && ($exclusions = explode("\n", $manual_exclusions))) foreach ($exclusions as $exclusion) if (strstr($url, $exclusion)) return true;
		return false;
	}

	// Set the default viewing languages.
	function determine_language() {
	  global $gengo_force_language_checking;
		if (!$this->languages) return;
		if (strstr($_SERVER['PHP_SELF'], 'wp-comments-post.php') || strstr($_SERVER['REQUEST_URI'], 'wp-trackback.php')) {
			$this->strict_links = false;
			return;
		} elseif (!$gengo_force_language_checking && $this->is_excluded_url($_SERVER['REQUEST_URI'])) {
			return;
		}
		if (preg_match('/^s=|&s=/', $_SERVER['QUERY_STRING'])) {
			$searching = true;
		}
		
		if (($query_string = $_SERVER['QUERY_STRING']) && (false !== ($start_position = strpos($query_string, "language=")))) {
		  $specified_codes = substr($query_string, $start_position + 9);
		  if (false !== ($end_position = strpos($specified_codes, "&"))) $specified_codes = substr($specified_codes, 0, $end_position);
			$specified_codes = explode(GENGO_LANGUAGE_DIVIDER, $specified_codes);
			foreach ($specified_codes as $code) {
				if ($this->is_defined_code($code)) {
					$codes[] = $code;
					$this->language_supplied = true;
				} else {
					$invalid_code = true;
				}
			}
			if ($invalid_code) {
				$_SERVER['REQUEST_URI'] = remove_query_arg('language', $_SERVER['REQUEST_URI']);
				if (true == $this->language_supplied) {
				  $code_string = implode(GENGO_LANGUAGE_DIVIDER, $codes);
					$_SERVER['REQUEST_URI'] = $this->append_link_language($_SERVER['REQUEST_URI'], $code_string, true);
				}
				header('Location: ' . $_SERVER['REQUEST_URI']);
				die();
			}
		} else {
			$request = ($searching) ? substr($_SERVER['REQUEST_URI'], 0, strpos($_SERVER['REQUEST_URI'], '?')) : trailingslashit($_SERVER['REQUEST_URI']);
			$position = strrpos(substr($request, 0, -1), '/');
			$this->url_appended_codes = substr($request, $position + 1, -1);
			$request_codes = explode(GENGO_LANGUAGE_DIVIDER, $this->url_appended_codes);
			if ((count($request_codes) > 1) || ($this->is_recognised_code($request_codes[0]))) {
				foreach ($request_codes as $code) {
					if ($this->is_defined_code($code)) {
						$codes[] = $code;
						$this->language_supplied = true;
					} else {
						$invalid_code = true;
					}
				}
				if ($invalid_code) {
					if (true == $this->language_supplied) $code_string = implode(GENGO_LANGUAGE_DIVIDER, $codes) . '/';
					$_SERVER['REQUEST_URI'] = substr($request, 0, $position + 1) . $code_string;
					header('Location: ' . $_SERVER['REQUEST_URI']);
					die();
				}
			}
		}

		if (!$codes) {
			if ($cookie = trim($_COOKIE[$this->language_cookie])) {
				$codes = explode(GENGO_LANGUAGE_DIVIDER, $cookie);
			} else {
				$reading_language = get_option('gengo_default_reading_language');
			  switch ($reading_language) {
					case 0:
						preg_match_all('([a-z\-]{2,}+)', $_SERVER['HTTP_ACCEPT_LANGUAGE'], $matches);
						if ($accepted_languages = $matches[0]) {
							foreach($accepted_languages as $accepted_language) {
							  if (false === ($position = strpos($accepted_language, '-'))) $position = strlen($accepted_language);
								if ($this->is_defined_code($user_agent_code = substr($accepted_language, 0, $position))) $codes[] = $user_agent_code;
							}
						}
						break;
						
						case -1:
						  foreach ($this->languages as $language_id => $entry) {
								$codes[] = $entry->code;
							}
							break;
							
						default:
							$codes[] = $this->languages[$reading_language]->code;
							break;
				}
				if ($codes) {
					$codes = array_unique($codes);
				} else {
					$codes[] = $this->languages[$this->blog_default_language_id]->code;
				}
			}

			if (!is_admin() && !$searching && $this->append_urls) {
				$code_string = implode(GENGO_LANGUAGE_DIVIDER, $codes);
			  if (strpos(get_settings('permalink_structure'), "index.php") !== false) {
					$slashed = trailingslashit($_SERVER['REQUEST_URI']);
					if ($slashed == substr(trailingslashit($this->site_home), -strlen($slashed))) $_SERVER['REQUEST_URI'] = $slashed . "index.php/";
			  }
				header('Location: ' . $this->append_link_language($_SERVER['REQUEST_URI'], $code_string, true));
				die();
			}
		}
		if (!is_admin()) $this->store_language_preference($codes);
		if ($this->language_supplied && !$this->append_urls) {
		  $location = $query_string ? remove_query_arg('language', $_SERVER['REQUEST_URI']) : substr($request, 0, $position + 1);
			if ("index.php/" == substr($location, -10)) $location = substr($location, 0, -10);
			header('Location: ' . $location);
			die();
		}
	}

	// Check if gengo is installed or needs upgrading.
	function is_installed() {
		return (get_option('gengo_version') == $this->version);
	}

	// Remove information from the database.
	function uninstall() {
		global $wpdb, $wp_rewrite;

		// Remove database tables.
		$wpdb->query("DROP TABLE $this->post2lang_table");
		$wpdb->query("DROP TABLE $this->language_table");
		$wpdb->query("DROP TABLE $this->summary_table");
		$wpdb->query("DROP TABLE $this->cat2syn_table");
		$wpdb->query("DROP TABLE $this->synblock_table");

		// Remove user options.
		$wpdb->query("DELETE FROM $wpdb->usermeta WHERE meta_key = 'gengo_default_language'");

		// Remove options.
		delete_option('gengo_blog_default_language');
		delete_option('gengo_version');
		delete_option('gengo_url_exclusions');
		delete_option('gengo_append_urls');
		delete_option('gengo_allow_multiread');

		// Flush the rewrite rules to remove the language appending.
		remove_filter('rewrite_rules_array');
		$wp_rewrite->flush_rules();

		header('Location: http://localhost/subversion/wordpress/wp-admin/plugins.php?action=deactivate&plugin=gengo/gengo.php');
		die();
	}

	// Add Gengo's tables to the database and create the blog default language option.
	function install($current_version = 0) {
		global $wpdb;

		require_once(ABSPATH . 'wp-admin/upgrade-functions.php');
		require_once(ABSPATH . GENGO_DIR . 'gengo_schema.php');

		dbdelta($schema);

		if ($current_version) {
		  global $table_prefix;

		  if ($current_version <= 0.5) {
			  // Consolidate post2sum and post2lang.
				$post2sum_table = $table_prefix . "post2sum";
			  if ($summaries = $wpdb->get_results("SELECT * FROM $post2sum_table")) foreach ($summaries as $sum) $wpdb->query("UPDATE $this->post2lang_table SET summary_group = $sum->summary_group WHERE post_id = $sum->post_id");
				$wpdb->query("DROP TABLE $post2sum_table");

				// Split synonyms into normalised form.
				$synonym_table = $table_prefix . "synonyms";
				$wpdb->query("INSERT INTO $this->cat2syn_table(cat_id, language_id, synonym, sanitised) SELECT item_id, language_id, synonym, sanitised FROM $synonym_table WHERE item_id != 0");
				$wpdb->query("INSERT INTO $this->synblock_table(block_name, language_id, text) SELECT item_type, language_id, synonym FROM $synonym_table WHERE item_id = 0");
				$wpdb->query("DROP TABLE $synonym_table");
			}
		}

		$this->set_defined_languages();

		add_option('gengo_blog_default_language', '', 'Sets the default language code for the blog.', 'yes');
		add_option('gengo_url_exclusions', '', 'URL language code exclusions.', 'yes');
		add_option('gengo_append_urls', 1, 'Append URLs.', 'yes');
		add_option('gengo_default_reading_language', 0, 'The language to show users when they visit for the first time.', 'yes');

		if ($this->mysql_int_version() > 40100) update_option('gengo_allow_multiread', 1, 'Controls whether users can read in multiple languages.', 'yes');
		else update_option('gengo_allow_multiread', 'x', 'Reading in multiple languages is disabled because your MySQL version is less than 4.1.', 'yes');

		update_option('gengo_version', $this->version, 'Current version of Gengo.', 'yes');

		$message = ($current_version) ? __('Gengo successfully upgraded.', GENGO_DOMAIN) : __('Gengo installed.', GENGO_DOMAIN);
		return $this->update_message($message);
	}

	function mysql_int_version() {
	  global $wpdb;
	  $version = (explode('.', $wpdb->get_var('SELECT VERSION()')));
	  return (int) sprintf('%d%02d%02d', $version[0], $version[1], intval($version[2]));
	}

	// Return the user's default language.
	function get_default_language_id() {
		global $current_user;
	 	if ($default = get_usermeta($current_user->id, 'gengo_default_language')) return $default;
	 	else return $this->blog_default_language_id;
	}

	// Return the blog default language.
	function set_blog_default_language() {
		$this->blog_default_language_id = get_option('gengo_blog_default_language');
	}

	// Set all posts with no language to the default language.
	function set_no_lang_posts_default($silent = false) {
		global $wpdb;
		if (!$default = $this->blog_default_language_id) return $this->error_message(__("Please set the default language first.", GENGO_DOMAIN));

		// Add entries to the language table for all those posts that don't have an entry.
		$wpdb->query("INSERT INTO $this->post2lang_table(post_id, language_id, translation_group, summary_group) SELECT p.ID, 1, '', '' FROM $wpdb->posts AS p LEFT JOIN $this->post2lang_table AS p2l ON p2l.post_id = p.ID WHERE p2l.post_id IS NULL;");

		$language = $this->languages[$default]->language;
		if (!$silent) $this->update_message(sprintf(__("Posts with no language have been set to '%s'.", GENGO_DOMAIN), $language));
		return true;
	}

	// Set a user's default language.
	function set_default_language($id) {
		global $current_user;
		$language = $this->languages[$id]->language;
		update_usermeta($current_user->id, 'gengo_default_language', $id);
		return $this->update_message(sprintf(__("Default language set to '%s'.", GENGO_DOMAIN), $language));
	}

	// Set the blog default language.
	function save_blog_default_language($id) {
		$language = $this->languages[$id]->language;
		update_option('gengo_blog_default_language', $id);
		$this->set_blog_default_language($id);
		return $this->update_message(sprintf(__("Default blog language set to '%s'.", GENGO_DOMAIN), $language));
	}

	// Return language data associated with a post.
	// Accepts a comma delimited string of fields to query.
	function get_post_language_data($post_id, $fields) {
		global $wpdb;
		if (count(explode(',', $fields)) > 1) return $wpdb->get_row("SELECT $fields FROM $this->post2lang_table AS p2l INNER JOIN $this->language_table AS l ON p2l.language_id = l.language_id WHERE post_id = $post_id LIMIT 1", ARRAY_A);
		else return $wpdb->get_var("SELECT $fields FROM $this->post2lang_table AS p2l INNER JOIN $this->language_table AS l ON p2l.language_id = l.language_id WHERE post_id = $post_id LIMIT 1");
	}

	// Fetch an array of languages and codes from the database.
	function set_defined_languages() {
		global $wpdb;
		$this->languages = array();
		$wpdb->hide_errors();
		if ($defined_languages = $wpdb->get_results("SELECT * FROM $this->language_table;", ARRAY_A)) {
			foreach ($defined_languages as $defined_language) {
				extract ($defined_language, EXTR_OVERWRITE);
				$this->languages[$language_id] = new Language($code, $language, $locale, $rtl);
				$this->codes2ids[$code] = $language_id;
			}
		}
		$wpdb->show_errors();
	}

	// Return counts for the number of posts, pages or all of a certain language.
	function get_total_by_language_id($id, $type = 'all') {
		global $wpdb;
		if (!$this->languages) return 0;
		switch ($type) {
			case 'all':
		    return $wpdb->get_var("SELECT COUNT(*) FROM $this->post2lang_table WHERE language_id = '$id'");

			case 'post':
				return $wpdb->get_var("SELECT COUNT(*) FROM $this->post2lang_table AS p2l INNER JOIN $wpdb->posts AS p ON p.ID = p2l.post_id WHERE p2l.language_id = '$id' AND p.post_status = 'publish'");

			case 'page':
				return $wpdb->get_var("SELECT COUNT(*) FROM $this->post2lang_table AS p2l INNER JOIN $wpdb->posts AS p ON p.ID = p2l.post_id WHERE p2l.language_id = '$id' AND p.post_status = 'static'");
		}
		return 0;
	}

	// Returns one complete translation link.
	function translation_link($translation_data, $type = 'a') {
		global $wp_rewrite;

		$post_id = $translation_data['post_id'];
		$code = $this->languages[$translation_data['language_id']]->code;
		$language = $this->languages[$translation_data['language_id']]->language;
		switch ($type) {
			case 'link':
				return "<link rel=\"alternate\" title=\"" . sprintf(__('This page in %s.', GENGO_DOMAIN), $language) . "\" type=\"text/html\" hreflang=\"$code\" href=\"" . get_permalink($post_id) . "\" />";

			case 'a':
				return "<a class=\"gengo_lang_$code\" rel=\"alternate\" hreflang=\"$code\" href=\"" . get_permalink($post_id) . "\" title=\"" . sprintf(__('View the %s translation of this article.', GENGO_DOMAIN), $language) . "\">$language</a>";
				
			case 'comments':
				return "<a class=\"gengo_lang_$code\" rel=\"alternate\" hreflang=\"$code\" href=\"" . get_permalink($post_id) . "#comments\" title=\"" . sprintf(__('View the %s comments for this article.', GENGO_DOMAIN), $language) . "\">$language</a> ";
				
			case 'home_link':
				$home = $this->site_home . '/';
				if ($wp_rewrite->using_index_permalinks()) $home .= $wp_rewrite->index . '/';
				$home = $this->append_link_language($home, $code);
				return "<link rel=\"alternate\" title=\"" . sprintf(__('This page in %s.', GENGO_DOMAIN), $language) . "\" type=\"text/html\" hreflang=\"$code\" href=\"$home\" />";
		}
		return '';
	}

	// Given a post_id, returns an array of posts that are a translation of this one.
	// With no post_id, assumes we are in The Loop and uses $post->translation_group.
	function get_the_translations($id = 0) {
		global $wpdb, $post;
		$translation_group = ($id) ? $wpdb->get_var("SELECT translation_group FROM $this->post2lang_table WHERE post_id = $id LIMIT 1") : $post->translation_group;
		if ($translation_group) {
			if ($translations = $wpdb->get_results("SELECT language_id, post_id FROM $this->post2lang_table AS p2l INNER JOIN $wpdb->posts AS p ON p.ID = p2l.post_id WHERE p2l.translation_group = $translation_group AND p2l.post_ID != $post->ID AND p.post_status != 'draft' ORDER BY language_id", ARRAY_A)) {
				return $translations;
			}
		}
		return array();
	}

	// Get all the summaries in a specified group.
	function get_the_summaries_by_summary_group($summary_group) {
		global $wpdb;
		if (!$this->languages) return array();
		if ($summaries = $wpdb->get_results("SELECT language_id, summary_id, summary, summary_group FROM $this->summary_table WHERE summary_group = $summary_group", ARRAY_A)) {
			return $summaries;
		}
		return array();
	}

	// Given a post_id, returns an array of summaries of that post.
	function get_the_summaries($post_id = false) {
		global $wpdb, $post;
		if (!$post_id) $post_id = $post->ID;
		if (!$post_id || !$this->languages) return array();
		if (is_array($post_id)) $post_id = implode(',', $post_id);
		if ($summaries = $wpdb->get_results("SELECT post_id, s.language_id AS language_id, summary_id, summary, s.summary_group AS summary_group FROM $this->post2lang_table p2l INNER JOIN $this->summary_table s ON s.summary_group = p2l.summary_group WHERE post_id IN ($post_id) ORDER BY language_id, summary_group", ARRAY_A)) {
			return $summaries;
		}
		return array();
	}

	// Ajax Server Functions.

	// Return post content for the second translation pane.
	function get_translation_content($post_id) {
		global $wpdb;
		$readonly = !current_user_can('edit_post', $post_id);
		$post_data = $wpdb->get_row("SELECT post_content, post_title, l.language_id FROM $wpdb->posts p INNER JOIN $this->post2lang_table p2l ON p.ID = p2l.post_id INNER JOIN $this->language_table l ON p2l.language_id = l.language_id WHERE ID = $post_id LIMIT 1", ARRAY_A);
		echo $post_data['language_id'] . "GENGO_SEPARATOR" . $post_data['post_title'] . "GENGO_SEPARATOR" . $readonly . "GENGO_SEPARATOR" . $post_data['post_content'];
	}

	// Return post content for the second translation pane.
	function get_summary_content($summary_id) {
		global $wpdb;
		$summary_data = $wpdb->get_row("SELECT language_id, summary FROM $this->summary_table WHERE summary_id = $summary_id", ARRAY_A);
		echo $summary_data['language_id'] . "GENGO_SEPARATOR" . $summary_data['summary'];
	}

	// Save the translation from the second translation pane.
	function update_translation_content($translation_id, $translation_content) {
		global $wpdb;
		if (!current_user_can('unfiltered_html')) $translation_content = wp_filter_post_kses($translation_content);
		$wpdb->query("UPDATE $wpdb->posts SET post_content = '$translation_content', post_modified = '" . current_time('mysql') . "', post_modified_gmt = '" . current_time('mysql', true) . "' WHERE ID = $translation_id");
	}

	// Update a summary with the given content.
	function update_summary_content($summary_id, $summary_content) {
		global $wpdb;
		if (!current_user_can('unfiltered_html')) $summary_content = wp_filter_post_kses($summary_content);
		$wpdb->query("UPDATE $this->summary_table SET summary = '$summary_content' WHERE summary_id = $summary_id");
	}

	// Add a new summary to a post.
	function add_summary_content($post_id, $summary_group, $language_id, $summary_content, $translation_group) {
		global $wpdb;
		if (!current_user_can('unfiltered_html')) $summary_content = wp_filter_post_kses($summary_content);
		if (!$summary_group) $summary_group = $wpdb->get_var("SELECT MAX(summary_group) FROM $this->post2lang_table") + 1;
		$wpdb->query("INSERT INTO $this->summary_table(summary_group, language_id, summary) VALUES ($summary_group, $language_id, '$summary_content')");
		$posts = ($translation_group && ($post_ids = $wpdb->get_col("SELECT post_id FROM $this->post2lang_table WHERE translation_group = $translation_group"))) ? implode(',', $post_ids) : $post_id;
		$wpdb->query("UPDATE $this->post2lang_table SET summary_group = $summary_group WHERE post_id IN ($posts)");
		echo $summary_group;
	}

	// Deletes a summary.
	function delete_summary($summary_id, $summary_group) {
		global $wpdb;

		// Remove the summary.
		$wpdb->query("DELETE FROM $this->summary_table WHERE summary_id = $summary_id");

		// Remove the summary group if this was the last one.
		if (!$wpdb->get_var("SELECT COUNT(*) FROM $this->summary_table WHERE summary_group = $summary_group")) {
			$wpdb->query("UPDATE $this->post2lang_table SET summary_group = 0 WHERE summary_group = $summary_group");
		}
	}

	// Prints a single block summary.
	function print_summary($summary_info, $highlight, $edit, $max_width = 55, $check = false) {
	  extract($summary_info, EXTR_OVERWRITE);
		if ('clash' == $highlight) {
			$background = '#ffebeb';
		} elseif ('added' == $highlight) {
			$background = '#ebffeb';
		} else {
			$background = '#e9e9e9';
		}
		?>
		<p id="sum_block_<?php echo $summary_id ?>" style="font-size: 11px; background: <?php echo $background ?>;">
		<?php
		if ($edit) {
			?>
			<a href="" onclick="gengo_delete_summary(<?php echo "$summary_id, $summary_group"; ?>); return false;" style="float: right; color: #000; margin-bottom: -2px;"><?php _e('Delete', GENGO_DOMAIN) ?></a>
			<a href="" onclick="gengo_show_summary_content_block(<?php echo $summary_id ?>); return false" style="float: right; margin-right: 1px; color: #000; margin-bottom: -2px;"><?php _e('Edit', GENGO_DOMAIN) ?></a>
			<?php
		}
		elseif (('clash' == $highlight) && $check) {
			?>
			<input type="radio" name="lang_clash_<?php echo $language_id ?>" value="<?php echo $summary_id ?>" style="float: right; margin: 0" onclick="gengo_resolve_clashes(<?php echo $language_id ?>)" />
			<?php
		}
		printf(__('in %s:', GENGO_DOMAIN), $this->languages[$language_id]->language);
		?>
		<br />
		<em><?php echo (strlen($summary) > $max_width) ? substr($summary, 0, $max_width - 2) . ".." : $summary; ?></em>
		</p>
		<?php
	}

	function list_summaries($summaries, & $excluded, $highlight = 'existing', $edit = true) {
		$excluded = array();
		foreach ($summaries as $sum) {
		  $this->print_summary($sum, $highlight, $edit);
			$excluded[] = $sum['language_id'];
		}
	}

	function get_group_components($posts) {
	  global $wpdb;

		if (!$summaries = $this->get_the_summaries($posts)) return;

		if (is_array($posts)) $posts = implode(',', $posts);
		$post_languages = $wpdb->get_results("SELECT post_id, language_id FROM $this->post2lang_table WHERE post_id in ($posts)", ARRAY_A);
		foreach ($post_languages as $entry) $lookup[$entry['post_id']] = $entry['language_id'];

		foreach ($summaries as $key => $sum) {
		  if ($key) echo "GENGO_SEPARATOR";
			echo $lookup[$sum['post_id']] . '_' . $sum['language_id'] . "GENGO_SEPARATOR";
			if (($summaries[$key + 1]['language_id'] == $sum['language_id']) || ($summaries[$key - 1]['language_id'] == $sum['language_id'])) {
				$this->print_summary($sum, 'clash', false, 55, true);
			  echo "GENGO_SEPARATOR" . $sum['language_id'];
			}
			else {
				$this->print_summary($sum, 'existing', false);
				?>
				<input type="hidden" name="gengo_summary_ids[]" value="<?php echo $sum['summary_id'] ?>" />
				<?php
			  echo "GENGO_SEPARATOR0";
			}
		}
	}

	function generate_summary_lists($post_id, $existing_summary_group, $is_in_group, $translation_number = 0) {
		global $wpdb;

		$post_summaries = $this->get_the_summaries_by_summary_group($existing_summary_group);

		if ($translation_number) {
			if ('true' == $is_in_group) {
				if (!$summary_group = $wpdb->get_var("SELECT summary_group FROM $this->post2lang_table WHERE translation_group = $translation_number LIMIT 1")) {
					$summary_group = 0;
				}
				if ($summary_group != $existing_summary_group) {
					$translation_summaries = $this->get_the_summaries_by_summary_group($summary_group);
				}
			} else {
				$translation_summaries = $this->get_the_summaries($translation_number);
				if (!$summary_group = $wpdb->get_var("SELECT summary_group FROM $this->post2lang_table WHERE post_id = $translation_number LIMIT 1")) {
					$summary_group = 0;
				}
			}
		} elseif ('true' == $is_in_group) {
			$summary_group = $existing_summary_group;
		} else {
			$summary_group = 0;
		}

		$excluded = array();
		if ($summary_group == $existing_summary_group) {
			$this->list_summaries($post_summaries, $excluded);
		} else {
			// We have changed the summary group and this posts' summaries will be unlinked.
			$changed = true;
			if ($translation_summaries) {
				?>
				<p style="font-size: 11px;"><em><?php _e('The following summaries will be linked:', GENGO_DOMAIN) ?></em></p>
				<?php
				$this->list_summaries($translation_summaries, $excluded, 'added');
			}
			if ($post_summaries) {
				?>
				<p style="font-size: 11px;"><strong><?php _e('Warning: The following summaries will be unlinked.', GENGO_DOMAIN) ?></strong></p>
				<?php
				$this->list_summaries($post_summaries, $dummy, 'clash', false);
			}
		}
		foreach ($this->languages as $language_id => $entry) if (!in_array($language_id, $excluded)) $summary_language .= "<option value=\"$language_id\">$entry->language</option>";
		?>
		<input type="hidden" id="gengo_summary_group" name="gengo_summary_group" value="<?php echo $summary_group ?>" />
		<?php
		if ($changed) return;
		if ($post_id) {
			if ($summary_language) {
				?>
				<label for="gengo_new_summary" class="selectit"><?php _e('Add a summary:', GENGO_DOMAIN) ?></label>
				<select name="gengo_new_summary" id="gengo_new_summary"><?php echo $summary_language; ?></select>
				<p class="submit"><input style="float: right" type="button" value="<?php _e('Add Summary', GENGO_DOMAIN) ?>" onclick="gengo_show_summary_content_block(0)" /></p>
				<?php
			}
		} else {
			?>
			<p style="font-size: 11px"><?php _e('Please save the post before adding summaries.', GENGO_DOMAIN) ?></p>
			<?php
		}
	}

	// Return the possible translation options for a given post.
	// Elite SQL queries in this function after great advice from Rudy at r937.com
	function generate_translation_lists($post_language_id, $post_translation_group, $existing_post_id, $first_post = 0, $first_group = 0) {
		global $wpdb;

		// First display the current group if we are in a group and we haven't changed the language.
		if ($post_translation_group) {
			$existing_language_id = $this->get_post_language_data($existing_post_id, 'p2l.language_id');
			if ($existing_language_id == $post_language_id || !$wpdb->get_var("SELECT post_id FROM $this->post2lang_table WHERE language_id = $post_language_id AND translation_group = $post_translation_group LIMIT 1")) {
				?>
				<span style="float: right; color: #000; font-size: 11px;">(<?php _e('Current', GENGO_DOMAIN) ?>)</span>
				<label for="gengo_translation_group_<?php echo $post_translation_group ?>" class="selectit">
				<input type="radio" id="gengo_translation_group_<?php echo $post_translation_group ?>" name="gengo_translation_group" onclick="gengo_lock_controls(); gengo_refresh_summary_list();" value="<?php echo $post_translation_group ?>" checked="checked" />
				<?php _e('of this group:', GENGO_DOMAIN); ?></label>
				<p style="margin: 0 0 10px 0; background: #e9e9e9;">
				<?php
				$group_info = $wpdb->get_results("SELECT ID as post_id, post_title, translation_group FROM $this->post2lang_table p2l INNER JOIN $wpdb->posts p ON p.ID = p2l.post_id WHERE translation_group = $post_translation_group", ARRAY_A);
				foreach ($group_info as $group) {
					extract($group, EXTR_OVERWRITE);
					$title = (strlen($post_title) > GENGO_SIDEBAR_TITLE_LENGTH) ? substr($post_title, 0, GENGO_SIDEBAR_TITLE_LENGTH - 2) . ".." : $post_title;
					if ($existing_post_id != $post_id) {
						?>
						<a href="" onclick="gengo_show_translation_content_block(<?php echo $post_id ?>); return false;"><?php echo $title ?></a><br />
						<?php
					} else {
						echo $title?><br /><?php
					}
				}
				?>
				</p>
				<?php
			}
		}
		if ($inclusions = $wpdb->get_col("SELECT DISTINCT translation_group FROM $this->post2lang_table GROUP BY translation_group HAVING 0 = COUNT(CASE WHEN language_id = $post_language_id OR translation_group = 0 OR translation_group = $post_translation_group THEN 37337 END) ORDER BY translation_group DESC LIMIT $first_group, 3")) {
			$included_groups = implode(',', $inclusions);
			if ($group_info = $wpdb->get_results("SELECT p.ID AS post_id, p.post_title AS post_title, p2l.translation_group AS translation_group FROM $this->post2lang_table p2l INNER JOIN $wpdb->posts p ON p.ID = p2l.post_id WHERE translation_group IN ($included_groups) ORDER BY translation_group", ARRAY_A)) {
				foreach ($group_info as $group) {
					extract($group, EXTR_OVERWRITE);
					if ($translation_group != $previous_translation_group) {
						if ($previous_translation_group){
							?></p><?php
						}
						?>
						<label for="gengo_translation_group_<?php echo $translation_group ?>" class="selectit">
						<input type="radio" id="gengo_translation_group_<?php echo $translation_group ?>" name="gengo_translation_group" onclick="gengo_lock_controls(); gengo_refresh_summary_list();" value="<?php echo $translation_group ?>" />
						<?php _e('of this group:', GENGO_DOMAIN); ?></label>
						<p style="margin: 0 0 10px 0; background: #e9e9e9;">
						<?php
					}
					$previous_translation_group = $translation_group;
					$title = (strlen($post_title) > GENGO_SIDEBAR_TITLE_LENGTH) ? substr($post_title, 0, GENGO_SIDEBAR_TITLE_LENGTH - 2) . ".." : $post_title;
					?>
					<a href="" onclick="gengo_show_translation_content_block(<?php echo $post_id ?>); return false;"><?php echo $title ?></a><br />
					<?php
				}
				?>
				</p>
				<?php
			}
			?>
			<p class="submit" id="gengo_group_navigation">
			<?php
			if ($first_group) {
				$previous = ($first_group < GENGO_GROUPS_LIMIT) ? $first_group : GENGO_GROUPS_LIMIT;
				?>
				<input type="button" style="float: left" value="&laquo; Prev <?php echo $previous ?>" onclick="gengo_update_translation_options(<?php echo $first_post . ', ' . ($first_group - $previous) ?>)" />
				<?php
			}
			$group_count = count($wpdb->get_col("SELECT DISTINCT translation_group FROM $this->post2lang_table GROUP BY translation_group HAVING 0 = COUNT(CASE WHEN language_id = $post_language_id OR translation_group = 0 OR translation_group = $post_translation_group THEN 37337 END)"));
			if ($group_count > ($first_group + GENGO_GROUPS_LIMIT)) {
				$remaining = $group_count - ($first_group + GENGO_GROUPS_LIMIT);
				if ($remaining > GENGO_GROUPS_LIMIT) $remaining = GENGO_GROUPS_LIMIT;
				?>
				<input type="button" style="float: right" value="Next <?php echo $remaining ?> &raquo;" onclick="gengo_update_translation_options(<?php echo $first_post . ', ' . ($first_group + GENGO_GROUPS_LIMIT) ?>)" />
				<?php
			}
			?>
			</p>
			<?php
		}
		if ($post_info = $wpdb->get_results("SELECT DISTINCT p.ID AS post_id, p.post_title AS post_title FROM $wpdb->posts AS p INNER JOIN $this->post2lang_table p2l ON p.ID = p2l.post_ID WHERE p2l.language_id != $post_language_id AND p2l.post_id != $existing_post_id AND p2l.translation_group = 0 ORDER BY ID DESC LIMIT $first_post, " . GENGO_POSTS_LIMIT, ARRAY_A)) {
			if (!$group_info) {
				// Hack to force an array if we can only select a post for the translation.
				?>
				<input type="hidden" value="" name="gengo_translation_group" />
				<?php
			}
			foreach ($post_info as $info) {
				extract($info, EXTR_OVERWRITE);
				$title = (strlen($post_title) > GENGO_SIDEBAR_SELECT_LENGTH) ? substr($post_title, 0, GENGO_SIDEBAR_SELECT_LENGTH - 2) . ".." : $post_title;
				$translation_list .= "<option value=\"$post_id\">$title</option>";
			}
			?>
			<label for="gengo_post_translation" style="clear: both" class="selectit"><input type="radio" name="gengo_translation_group" id="gengo_post_translation" value="0"<?php if (!$post_translation_group) echo ' checked="checked"' ?> onclick="gengo_show_translation_content_block(document.getElementById('gengo_translation_post').value);"> <?php _e('of the following post/page:', GENGO_DOMAIN) ?></label>
			<select name="gengo_translation_post" id="gengo_translation_post" onchange="document.getElementById('gengo_post_translation').checked = true; gengo_show_translation_content_block(this.value);"><?php echo $translation_list; ?></select>
			<p class="submit" id="gengo_post_navigation">
			<?php
			if ($first_post) {
				$previous = ($first_post < GENGO_POSTS_LIMIT) ? $first_post : GENGO_POSTS_LIMIT;
				?>
				<input type="button" style="float: left" value="&laquo; Prev <?php echo $previous ?>" onclick="gengo_update_translation_options(<?php echo ($first_post - $previous) . ', ' . $first_group ?>)" />
				<?php
			}
			$post_count = $wpdb->get_var("SELECT COUNT(*) FROM $wpdb->posts AS p INNER JOIN $this->post2lang_table p2l ON p.ID = p2l.post_ID WHERE p2l.language_id != $post_language_id AND p2l.post_id != $existing_post_id AND p2l.translation_group = 0");
			if ($post_count > ($first_post + GENGO_POSTS_LIMIT)) {
				$remaining = $post_count - ($first_post + GENGO_POSTS_LIMIT);
				if ($remaining > GENGO_POSTS_LIMIT) $remaining = GENGO_POSTS_LIMIT;
				?>
				<input type="button" style="float: right" value="Next <?php echo $remaining ?> &raquo;" onclick="gengo_update_translation_options(<?php echo ($first_post + GENGO_POSTS_LIMIT) . ', ' . $first_group ?>)" />
				<?php
			}
			?>
			</p>
			<?php
		}
		if (!$post_info && !$group_info) {
		  $language = $this->languages[$post_language_id]->language;
			// Nasty hack to prevent javascript errors.
			?>
			<input type="hidden" value="" name="gengo_translation_group" />
			<p style="font-size: 11px;"><?php printf(__('No translation options available. Write a post or page in a language other than %s.', GENGO_DOMAIN), $language) ?></p>
			<?php
		}
	}
}

if (!is_admin()) require_once(ABSPATH . GENGO_DIR . 'gengo_template_functions.php');

// Handle AJAX Callbacks.
if (isset($_POST['gengo_ajax'])) {

	$gengo = new Gengo(true);
	switch ($_POST['gengo_action']) {
		case 'ajax_update_translation_options':
		$gengo->generate_translation_lists($_POST['post_language_id'], $_POST['post_translation_group'],  $_POST['post_id'], $_POST['first_post'], $_POST['first_group']);
		break;

		case 'ajax_get_translation_post':
		$gengo->get_translation_content($_POST['post_id']);
		break;

		case 'ajax_update_translation_content':
		$gengo->update_translation_content($_POST['translation_id'], $_POST['translation_content']);
		break;

		case 'ajax_get_summary_content':
		$gengo->get_summary_content($_POST['summary_id']);
		break;

		case 'ajax_refresh_summary_lists':
		$gengo->generate_summary_lists($_POST['post_id'], $_POST['summary_group'], $_POST['is_in_group'], $_POST['translation_number']);
		break;

		case 'ajax_update_summary_content':
		$gengo->update_summary_content($_POST['summary_id'], $_POST['summary_content']);
		break;

		case 'ajax_add_summary_content':
		$gengo->add_summary_content($_POST['post_id'], $_POST['summary_group'], $_POST['language_id'], $_POST['summary_content'], $_POST['translation_group']);
		break;

		case 'ajax_delete_summary':
		$gengo->delete_summary($_POST['summary_id'], $_POST['summary_group']);
		break;

		case 'ajax_get_group_components':
		$gengo->get_group_components($_POST['post_ids']);
		break;

		case 'ajax_get_synblock':
		$gengo->get_synblock($_POST['block_name']);
		break;
	}
	exit;
}

$gengo = new Gengo();

/* TODO:
Remove CSS from language control box and create a demonstration.

Future:
-------
separate Javascript so only those dynamically generated are created each time.  Cache the rest.
global wpdb => $this->db
?translation_group=23
prevent post slugs of code.
Smart summaries on the post screen. Major.
Store summary author in the database? Poll.
Merge translation and post group functions. Major.
Add post to existing translation group. Minor.
Summaries page. Major.
Change GETs to POSTs or look at nonces.
INSERT TABLE 'genericName' isn't allowed in FROM table list
dbx language control box.
prefill list of available languages with browser preference - bloatware?
Move category maintenance stuff to Category page.
Menu for Category descriptions.

For consideration after 2.1:
----------------------------
Use the script loader to cache JS.
Replace gengo_list_cats with get_categories() filter.
Replace gengo_list_pages better with get_pages() filter.
Auto update notification?  Matt Read + Red Alt?
Impact of splitting post_status into post_type.
Impact of new rewrite API.
Previous next links by language.
Better roles and capabilities.
Translate bookmarks.
Revisit feeds.
WYSIWYG x2 Can't be easily done yet.  Significant refactoring to come in 2.1
Combined translation comments.
Move locale switching back to locale filter if we can filter more than once.
Add endpoint to trackback url.
Redo previous and next page links to use _max_num_pages().

*/
?>