<?php
// Returns a list of defined languages as a formatted list.
function gengo_list_languages($show_count = true, $before = '<li>', $after = '</li>', $show_unreadable = true) {
	global $gengo, $wp_rewrite;

	if (!$gengo->languages) {
		echo $before . __('No Languages Defined', GENGO_DOMAIN) . $after;
	} else {
		$home = trailingslashit(get_settings('home'));
		if ($wp_rewrite->using_index_permalinks()) $home .= $wp_rewrite->index;
		foreach ($gengo->languages as $language_id => $entry) {
			if ($post_count = $gengo->get_total_by_language_id($language_id, 'post')) {
			  if ($show_count) $count_string = " ($post_count)";
			  $gengo->forced_append = true;
			  if ($show_unreadable || (false !== strpos($gengo->viewable_code_string, $entry->code))) {
					if (false === $gengo->is_set_language($entry->code)) echo "$before<a class=\"gengo_lang_$entry->code\" href=\"" . $gengo->append_link_language($home, $entry->code) . "\">$entry->language</a>$count_string$after";
					else echo "$before<span class=\"gengo_lang_$entry->code\">$entry->language$count_string</span>$after";
				}
			}
		}
	}
}

function gengo_list_pages($args = '')
{
	global $gengo, $wpdb;

	if ($code = $gengo->language_preference[0]->code)
	{
	  if (1 == count($gengo->language_preference)) $where = "l.code = '$code'";
	  else
		{
			foreach ($gengo->language_preference_id as $key => $language_id)
			{
				if (0 == $key) $where = "l.language_id = $language_id";
				else
				{
					$zero_groups .= $zero_groups ? ",$language_id" : $language_id;
					$where .= " OR (l.language_id = $language_id AND translation_group IN (SELECT translation_group FROM $gengo->post2lang_table GROUP BY translation_group HAVING 0 = COUNT(CASE WHEN language_id IN ($exclusions) THEN 37337 END)))";
				}
				$exclusions .= ($exclusions) ? ",$language_id" : $language_id;
			}
			if ($zero_groups) $where .= " OR (l.language_id IN ($zero_groups) AND translation_group = 0)";
		}

		if ($ids = $wpdb->get_col("SELECT ID FROM $wpdb->posts WHERE ID NOT IN (SELECT p2l.post_id FROM $gengo->post2lang_table p2l INNER JOIN $wpdb->posts p ON p.ID = p2l.post_id INNER JOIN $gengo->language_table l ON p2l.language_id = l.language_id WHERE $where AND p.post_status = 'static')"))
		{
			parse_str($args, $r);
			$r['exclude'] = implode(',', array_unique(array_merge(explode(',', $r['exclude']), $ids)));
			foreach ($r as $arg => $val) $arr[] = "$arg=$val";
			$args = implode($arr, '&');
		}
	}
	return wp_list_pages($args);
}

function gengo_list_cats($args = '')
{
	global $gengo, $wpdb;

	parse_str($args, $r);
	if ( !isset($r['optionall']))
		$r['optionall'] = 0;
	if ( !isset($r['all']))
		$r['all'] = 'All';
	if ( !isset($r['sort_column']) )
		$r['sort_column'] = 'ID';
	if ( !isset($r['sort_order']) )
		$r['sort_order'] = 'asc';
	if ( !isset($r['file']) )
		$r['file'] = '';
	if ( !isset($r['list']) )
		$r['list'] = true;
	if ( !isset($r['optiondates']) )
		$r['optiondates'] = 0;
	if ( !isset($r['optioncount']) )
		$r['optioncount'] = 0;
	if ( !isset($r['hide_empty']) )
		$r['hide_empty'] = 1;
	if ( !isset($r['use_desc_for_title']) )
		$r['use_desc_for_title'] = 1;
	if ( !isset($r['children']) )
		$r['children'] = true;
	if ( !isset($r['child_of']) )
		$r['child_of'] = 0;
	if ( !isset($r['recurse']) )
		$r['recurse'] = 0;
	if ( !isset($r['feed']) )
		$r['feed'] = '';
	if ( !isset($r['feed_image']) )
		$r['feed_image'] = '';
	if ( !isset($r['exclude']) )
		$r['exclude'] = '';
	if ( !isset($r['hierarchical']) )
		$r['hierarchical'] = true;

	$exclusions = $r['exclude'] ? 'AND p2c.category_id NOT IN (' . $r['exclude'] . ')' : '';
	if ($code = $gengo->language_preference[0]->code)
	{
		$viewable_codes = explode(GENGO_LANGUAGE_DIVIDER, $gengo->viewable_code_string);
		foreach ($viewable_codes as $viewable_code) $language_ids .= ($language_ids) ? ',' . $gengo->codes2ids[$viewable_code] : $gengo->codes2ids[$viewable_code];
		if ($language_ids && $category_results = $wpdb->get_results("SELECT p2c.category_id, COUNT(*) as category_count FROM $gengo->post2lang_table p2l INNER JOIN $wpdb->post2cat p2c ON p2c.post_id = p2l.post_id INNER JOIN $wpdb->posts p ON p2l.post_id = p.ID WHERE p2l.language_id IN ($language_ids) AND p.post_status = 'publish' $exclusions GROUP BY p2c.category_id;"))
		{
			foreach ($category_results as $cat)
			{
				$cats .= $cats ? ", $cat->category_id" : $cat->category_id;
				$counts[$cat->category_id] = $cat->category_count;
			}

			// Get the category information we want then iterate over the object and
			// assign the correct category count.
			$sort_column = 'ORDER BY cat_' . $r['sort_column'];
			$sort_order = $r['sort_order'];
			$language_cats = $wpdb->get_results("SELECT cat_ID, cat_name, category_nicename, category_description, category_parent, category_count FROM $wpdb->categories WHERE cat_ID IN ($cats) $sort_column $sort_order");
			while (list($key) = each($language_cats))
			{
				$cat = & $language_cats[$key];
				$cat->category_count = $counts[$cat->cat_ID];
   			unset($cat);
			}
		}
		else
		{
			?>
			<p><?php printf(__('No categories containing posts in %s.', GENGO_DOMAIN), the_viewable_languages(__('or', GENGO_DOMAIN), true)); ?></p>
			<?php
			return;
		}
	}
	return list_cats($r['optionall'], $r['all'], $r['sort_column'], $r['sort_order'], $r['file'], $r['list'], $r['optiondates'], $r['optioncount'], $r['hide_empty'], $r['use_desc_for_title'], $r['children'], $r['child_of'], $language_cats, $r['recurse'], $r['feed'], $r['feed_image'], $r['exclude'], $r['hierarchical']);
}

// A language-specific replacement for single_cat_title().
function gengo_single_cat_title($return = false)
{
	global $gengo, $wpdb, $post;

	if ($cat = intval(get_query_var('cat')) && !(strtoupper($cat) == 'ALL'))
	{
		if (!$title = $wpdb->get_var("SELECT synonym FROM $gengo->cat2syn_table WHERE cat_id = $cat_id AND language_id = $post->language_id LIMIT 1"))
			$title = $wpdb->get_var("SELECT cat_name FROM $wpdb->categories WHERE cat_ID = $cat_id LIMIT 1");
	}
  if ($return) return $title;
  else echo $title;
}

// Modified version of WordPress' own link_pages.
function gengo_link_pages($before = '<br />', $after = '<br />', $next_or_number = 'number', $nextpagelink = 'next page', $previouspagelink = 'previous page', $pagelink = '%', $more_file = '')
{
 	global $gengo, $id, $page, $numpages, $multipage, $more, $pagenow;

	if ($multipage)
	{
		$file = $more_file ? $more_file : $pagenow;
		$permalinks = get_settings('permalink_structure');

		if ($permalinks) $link_base = ($gengo->language_supplied) ? substr(trailingslashit(get_permalink()), 0, -3) : get_permalink();
		else $link_base = get_permalink() . '&amp;page=';

		if ('number' == $next_or_number)
		{
			echo $before;
 			for ($i = 1; $i < ($numpages + 1); $i++)
			{
				$j = str_replace('%', "$i", $pagelink);
				if (($i != $page) || ((!$more) && ($page == 1)))
				{
					$link = $link_base . $i;
					if ($gengo->language_preference && $permalinks) $link = $gengo->append_link_language($link, $gengo->language_preference[0]->code);
					echo "<a href=\"$link\">$j</a> ";
				}
				else echo "$j ";
			}
			echo $after;
		}
		elseif ($more)
		{
			echo $before;
			$i = $page - 1;
			if ($i && $more)
			{
				$link = $link_base . $i;
				if ($gengo->language_preference && $permalinks) $link = $gengo->append_link_language($link, $gengo->language_preference[0]->code);
				echo "<a href=\"$link\">$previouspagelink</a>";
			}
			$i = $page + 1;
			if ($i <= $numpages && $more)
			{
				$link = $link_base . $i;
				if ($gengo->language_preference && $permalinks) $link = $gengo->append_link_language($link, $gengo->language_preference[0]->code);
				echo "<a href=\"$link\">$nextpagelink</a>";
			}
			echo $after;
		}
	}
}

// Modified version of WordPress' own previous_posts_link.
function gengo_previous_posts_link($label = '&laquo; Previous Page')
{
	global $gengo, $paged;

	if (!is_single() && ($paged > 1))
	{
		$nextpage = intval($paged) - 1;
		if ($nextpage < 1) $nextpage = 1;
		$link = get_pagenum_link($nextpage);
		if ($gengo->language_preference) $link = str_replace("/{$gengo->viewable_code_string}/page/$nextpage/", "/page/$nextpage/{$gengo->viewable_code_string}/", $link);
		echo '<a href="' . $link . '">' . preg_replace('/&([^#])(?![a-z]{1,8};)/', '&#038;$1', $label) . '</a>';
	}
}

// Modified version of WordPress' own next_posts_link.
function gengo_next_posts_link($label = 'Next Page &raquo;', $max_page = 0)
{
	global $gengo, $paged, $max_num_pages, $result, $request, $posts_per_page, $wpdb;

	if (!$max_page)
	{
	  if (isset($max_num_pages)) $max_page = $max_num_pages;
	  else
	  {
	    $start_pos = strpos($request, 'FROM');
	    $end_pos = strrpos($request, 'GROUP BY');
			$numposts = $wpdb->get_var('SELECT COUNT(DISTINCT ID) ' . substr($request, $start_pos, $end_pos - $start_pos));
			$max_page = $max_num_pages = ceil($numposts / $posts_per_page);
		}
	}
	if (!$paged) $paged = 1;
	$nextpage = intval($paged) + 1;

	if (!is_single() && ($nextpage <= $max_page))
	{
		$link = get_pagenum_link($nextpage);
		foreach ($gengo->language_preference_id as $id) $string .= ($string) ? GENGO_LANGUAGE_DIVIDER . $gengo->languages[$id]->code : $gengo->languages[$id]->code;
		if ($gengo->language_preference) $link = str_replace("/$string/page/$nextpage/", "/page/$nextpage/$string/", $link);
		echo '<a href="' . $link . '">' . preg_replace('/&([^#])(?![a-z]{1,8};)/', '&#038;$1', $label) . '</a>';
	}
}

function gengo_synblock($block_name, $return = false)
{
	global $gengo;

	if ($code = $gengo->language_preference[0]->code)
	{
		if ($return) return $gengo->get_synblock($block_name, $gengo->language_preference_id[0]);
		else echo $gengo->get_synblock($block_name, $gengo->language_preference_id[0]);
	}
}

function gengo_trackback_url($display = true)
{
	global $gengo, $id, $post;

	$url = get_settings('permalink_structure') ? substr(trailingslashit(get_permalink()), 0, -3) . 'trackback/' : get_settings('siteurl') . '/wp-trackback.php?p=' . $id;
	$url = $gengo->append_link_language($url, $post->code);

	if ($display) echo $url;
	else return $url;
}

function gengo_viewing_languages($title = 'GENGO_DEFAULT')
{
	global $gengo;

	if ('GENGO_DEFAULT' == $title) $title = '<h2>' . __('Current Languages', GENGO_DOMAIN) . '</h2>';
	echo $title;
	?>
	<ul id="gengo_viewing_languages">
	<?php
	$viewing_languages = explode(GENGO_LANGUAGE_DIVIDER, $gengo->viewable_code_string);
	foreach ($viewing_languages as $code)
	{
	  $id = $gengo->codes2ids[$code];
		?>
		<li class="gengo_language_element" id="gengo_language_<?php echo $id ?>">
		<span class="gengo_language_element_name"><?php echo $gengo->languages[$id]->language ?></span>
		<span class="gengo_element_control_set" id="gengo_control_set_<?php echo $id ?>">
		<a class="gengo_element_control" href="#" onclick="return gengo_language_move_up(<?php echo $id ?>);"><?php _e('Up', GENGO_DOMAIN) ?></a>
		<a class="gengo_element_control" href="#" onclick="return gengo_language_move_down(<?php echo $id ?>);"><?php _e('Down', GENGO_DOMAIN) ?></a>
		<a class="gengo_element_control" href="#" onclick="return gengo_language_remove(<?php echo $id ?>);"><?php _e('x', GENGO_DOMAIN) ?></a>
		</span>
		</li>
		<?php
	}
	?>
	</ul>
	<?php
}

function gengo_available_languages($title = 'GENGO_DEFAULT')
{
	global $gengo;

	if ('GENGO_DEFAULT' == $title) $title = '<h2>' . __('Available Languages', GENGO_DOMAIN) . '</h2>';
	echo $title;
	?>
	<ul id="gengo_available_languages">
	<?php
	$viewing_languages = explode(GENGO_LANGUAGE_DIVIDER, $gengo->viewable_code_string);
	foreach ($gengo->languages as $language_id => $entry)
	{
	  if (!in_array($entry->code, $viewing_languages))
	  {
		?>
		<li class="gengo_language_element" id="gengo_language_<?php echo $language_id ?>">
		<span class="gengo_language_element_name"><?php echo $entry->language ?></span>
		<span class="gengo_element_control_set" id="gengo_control_set_<?php echo $language_id ?>">
		<a class="gengo_element_control" href="#" onclick="return gengo_language_add(<?php echo $language_id ?>);"><?php _e('Add', GENGO_DOMAIN) ?></a>
		</span>
		</li>
		<?php
		}
	}
	?>
	</ul>
	<?php
}

function gengo_language_set()
{
	global $gengo;
	?>
	<div class="gengo_settings_control_set">
	<a class="gengo_settings_save_control" href="#" onclick="return gengo_language_save();"><?php _e('Save', GENGO_DOMAIN) ?></a>
	<a class="gengo_settings_reset_control" href="#" onclick="return gengo_language_reset();"><?php _e('Reset', GENGO_DOMAIN) ?></a>
	</div>
	<?php
}

function gengo_language_control($viewing_title = 'GENGO_DEFAULT', $available_title = 'GENGO_DEFAULT')
{
	?>

	<div id="gengo_language_control">
	<?php
	gengo_viewing_languages($viewing_title);
	gengo_available_languages($available_title);
	gengo_language_set()
	?>
	</div>
	<?php
}

// Echoes or returns a home URL link with correct language appending.
function gengo_home_url($return = false)
{
	global $gengo, $wp_rewrite;

	if ($wp_rewrite->using_index_permalinks()) $index = $wp_rewrite->index;
	$url = $gengo->append_link_language(trailingslashit(get_settings('home')) . $index, $gengo->viewable_code_string);
	if ("index.php" == substr($url, -9)) $url = substr($url, 0, -9);

	if ($return) return $url;
	else echo $url;
}


// Checks whether we are on a language page.
function is_language($specific = '')
{
	global $gengo;

	if (!$code = $gengo->language_preference[0]->code) return false;
	if ($specific)
	{
		if (false !== $gengo->is_set_language($specific)) return true;
		return false;
	}
	return true;
}

// Echoes the language of the current view.
function the_language($return = false)
{
	global $post;

	if ($return) return $post->language;
	else echo $post->language;
}

// Echoes or returns the language code.  Thanks to Martin Gottstein for the idea.
function the_language_code($return = false)
{
	global $post;

	if ($return) return $post->code;
	else echo $post->code;
}

// Echoes or returns the language locale.
function the_language_locale($return = false)
{
	global $post;

	if ($return) return $post->locale;
	else echo $post->locale;
}

// Echoes or returns the language text direction
function the_language_direction($return = false)
{
	global $post;

	if ($return) return $post->rtl ? 'rtl' : 'ltr';
	else echo $post->rtl ? 'rtl' : 'ltr';
}

function the_viewable_languages($aspect, $return = false)
{
	global $gengo;

	$viewable_count = count($gengo->language_preference_id);
	if (1 == $viewable_count) $string = $gengo->languages[$gengo->language_preference_id[0]]->language;
	else
	{
		$i = 0;
		foreach ($gengo->language_preference_id as $id)
		{
		  if ($i++) $string .= ($i == $viewable_count) ? " $aspect " : ", ";
		  $string .= $gengo->languages[$id]->language;
		}
	}

	if ($return) return $string;
	else echo $string;
}

// Outputs a div containing all the summaries for this article.  Use in The Loop.
function the_summaries()
{
	global $gengo;

	if (!$summaries = $gengo->get_the_summaries()) return;
	$lowest = 10000;
	$default_id = 0;

	foreach ($summaries as $sum)
	{
		if (false !== ($order = $gengo->is_viewable_language($sum['language_id'])))
		{
		  $viewable_summaries[] = $sum;
			if ($order < $lowest)
		  {
				$default_id = $sum['summary_id'];
		  	$lowest = $order;
			}
		}
	}
	if (!$default_id) $default_id = $summaries[0]['summary_id'];

	if (($sum_count = count($viewable_summaries)) > 1) {
		?><script type="text/javascript">var current_id = <?php echo $default_id ?>;</script><?php
	}
	?>
	<br />
	<legend id="gengo_summaries_title" for="gengo_summaries_container"><?php _e('Summaries:', GENGO_DOMAIN) ?></legend>
	<div id="gengo_summaries_container">
	<?php
	foreach ($viewable_summaries as $sum)
	{
		extract($sum, EXTR_OVERWRITE);
		?>
		<p class="gengo_summary_inner" id="gengo_summary_<?php echo $summary_id ?>" style="display: <?php echo ($summary_id == $default_id) ? 'block' : 'none' ?>" xml:lang="<?php echo $gengo->languages[$language_id]->code ?>" lang="<?php echo $gengo->languages[$language_id]->code ?>"><?php echo $summary ?></p>
		<?php
		$link_list .= '<a class="gengo_summary_link" id="gengo_summary_' . $summary_id . '" href="#" onclick="gengo_switch_summary(' . $summary_id . '); return false;">' . $gengo->languages[$language_id]->language . '</a> ';
	}
	if ($sum_count > 1) echo $link_list;
	?>
	<p style="clear: both"></p>
	</div>
	<?php
}

// Returns a formatted list of translations for this article. Use in The Loop.
function the_translations($arguments = '')
{
	global $gengo;

	$default_args = array('return' => false, 'pre' => '<ul><li>', 'post' => '</li></ul>', 'inner' => '</li><li>', 'title_none' => 'GENGO_DEFAULT', 'title_exists' => 'GENGO_DEFAULT', 'show_current' => false);
  parse_str($arguments, $args);
  $args = array_merge($default_args, $args);

	if (!$translations = $gengo->get_the_translations())
	{
		if ($args['return']) return array();
	  if ('GENGO_DEFAULT' == $args['title_none']) $args['title_none'] = '<p>' . __('No Translations', GENGO_DOMAIN) . '</p>';
	  echo $args['title_none'];
		return;
	}
	if ($args['return']) return $translations;
	if ('GENGO_DEFAULT' == $args['title_exists']) $args['title_exists'] = '<p>' . __('Other Languages:', GENGO_DOMAIN) . '</p>';
	echo $args['title_exists'] . $args['pre'];
	foreach ($translations as $translation)
	{
	  if ($i++) echo $args['inner'];
	  echo $gengo->translation_link($translation);
	}
	if ($args['show_current']) echo $args['inner'] . str_replace('%', the_language(true), $args['show_current']);
	echo $args['post'];
}

function the_translations_comments($show_count = true)
{
	global $gengo, $wpdb, $post;

	if (!$translations = $wpdb->get_results("SELECT language_id, post_id, comment_count FROM $gengo->post2lang_table AS p2l INNER JOIN $wpdb->posts AS p ON p.ID = p2l.post_id WHERE p2l.translation_group = $post->translation_group AND p2l.post_ID != $post->ID AND p.post_status != 'draft' AND comment_count > 0 ORDER BY language_id", ARRAY_A)) return;
	foreach ($translations as $translation)
	{
		$comments_string .= $gengo->translation_link($translation, 'comments');
		if ($show_count) $comments_string .= " ($translation[comment_count]) ";
	}
	if ($comments_string) printf(__('View Comments in: %s', GENGO_DOMAIN), $comments_string);
}

include_once(ABSPATH . GENGO_DIR . 'gengo_extra_functions.php');
?>